# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2005 Thomas Vander Stichele
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import sys
from common import gobject, gst, unittest, TestCase
import gc

class NewTest(TestCase):
    def testEOS(self):
        gst.info("creating new bin")
        b = gst.Bin()
        gst.info("creating new EOS message from that bin")
        m = gst.message_new_eos(b)
        gst.info("got message : %s" % m)

    def message_application_cb(self, bus, message):
        gst.info("got application message")
        self.got_message = True
        self.loop.quit()

    def testApplication(self):
        self.loop = gobject.MainLoop()
        gst.info("creating new pipeline")
        bin = gst.Pipeline()
        bus = bin.get_bus()
        bus.add_signal_watch()
        self.got_message = False
        bus.connect('message::application', self.message_application_cb)

        struc = gst.Structure("foo")
        msg = gst.message_new_application(bin, struc)
        # the bus is flushing in NULL, so we need to set the pipeline to READY
        bin.set_state(gst.STATE_READY)
        bus.post(msg)
        self.loop.run()
        bus.remove_signal_watch()
        bin.set_state(gst.STATE_NULL)
        self.failUnless(self.got_message == True)
        self.gccollect()

class TestCreateMessages(TestCase):

    def setUp(self):
        TestCase.setUp(self)
        self.element = gst.Bin()

    def tearDown(self):
        del self.element

    def testCustomMessage(self):
        # create two custom messages using the same structure
        s = gst.Structure("something")
        assert s != None
        e1 = gst.message_new_custom(gst.MESSAGE_APPLICATION, self.element, s)
        assert e1
        e2 = gst.message_new_custom(gst.MESSAGE_APPLICATION, self.element, s)
        assert e2

        # make sure the two structures are equal
        self.assertEquals(e1.structure.to_string(),
                          e2.structure.to_string())

    def testTagMessage(self):
        # Create a taglist
        t = gst.TagList()
        t['something'] = "else"
        t['another'] = 42

        # Create two messages using that same taglist
        m1 = gst.message_new_tag(self.element, t)
        assert m1
        m2 = gst.message_new_tag(self.element, t)
        assert m2

        # make sure the two messages have the same taglist
        t1 = m1.parse_tag()
        assert t1
        keys = t1.keys()
        keys.sort()
        self.assertEquals(keys, ['another', 'something'])
        self.assertEquals(t1['something'], "else")
        self.assertEquals(t1['another'], 42)
        t2 = m2.parse_tag()
        assert t2
        keys = t2.keys()
        keys.sort()
        self.assertEquals(keys, ['another', 'something'])
        self.assertEquals(t2['something'], "else")
        self.assertEquals(t2['another'], 42)

    def testTagFullMessage(self):
        if hasattr(gst.Message, 'parse_tag_full'):
            p = gst.Pad("blahblah", gst.PAD_SRC)
            # Create a taglist
            t = gst.TagList()
            t['something'] = "else"
            t['another'] = 42

            # Create two messages using that same taglist
            m1 = gst.message_new_tag_full(self.element, p, t)
            assert m1
            m2 = gst.message_new_tag_full(self.element, p, t)
            assert m2

            # make sure the two messages have the same taglist
            p1, t1 = m1.parse_tag_full()
            assert t1
            keys = t1.keys()
            keys.sort()
            self.assertEquals(p1, p)
            self.assertEquals(keys, ['another', 'something'])
            self.assertEquals(t1['something'], "else")
            self.assertEquals(t1['another'], 42)
            p2, t2 = m2.parse_tag_full()
            assert t2
            keys = t2.keys()
            keys.sort()
            self.assertEquals(p2, p)
            self.assertEquals(keys, ['another', 'something'])
            self.assertEquals(t2['something'], "else")
            self.assertEquals(t2['another'], 42)

    def testStepStartMessage(self):
        if hasattr(gst, 'message_new_step_start'):
            m = gst.message_new_step_start(self.element, True,
                                           gst.FORMAT_TIME, 42, 1.0,
                                           True, True)
            self.assertEquals(m.type, gst.MESSAGE_STEP_START)
            active, format, amount, rate, flush, intermediate = m.parse_step_start()
            self.assertEquals(active, True)
            self.assertEquals(format, gst.FORMAT_TIME)
            self.assertEquals(amount, 42)
            self.assertEquals(rate, 1.0)
            self.assertEquals(flush, True)
            self.assertEquals(intermediate, True)

    def testStepDoneMessage(self):
        if hasattr(gst, 'message_new_step_done'):
            m = gst.message_new_step_done(self.element, gst.FORMAT_TIME, 42,
                                          1.0, True, True, 54, True)
            self.assertEquals(m.type, gst.MESSAGE_STEP_DONE)

            fmt, am, rat, flu, inter, dur, eos = m.parse_step_done()
            self.assertEquals(fmt, gst.FORMAT_TIME)
            self.assertEquals(am, 42)
            self.assertEquals(rat, 1.0)
            self.assertEquals(flu, True)
            self.assertEquals(inter, True)
            self.assertEquals(dur, 54)
            self.assertEquals(eos, True)

    def testStructureChangeMessage(self):
        if hasattr(gst, 'message_new_structure_change'):
            p = gst.Pad("blah", gst.PAD_SINK)
            m = gst.message_new_structure_change(p,
                                                 gst.STRUCTURE_CHANGE_TYPE_PAD_LINK,
                                                 self.element, True)

            self.assertEquals(m.type, gst.MESSAGE_STRUCTURE_CHANGE)
            sct, owner, busy = m.parse_structure_change()
            self.assertEquals(sct, gst.STRUCTURE_CHANGE_TYPE_PAD_LINK)
            self.assertEquals(owner, self.element)
            self.assertEquals(busy, True)

    def testRequestStateMessage(self):
        if hasattr(gst, 'message_new_request_state'):
            m = gst.message_new_request_state(self.element, gst.STATE_NULL)
            self.assertEquals(m.type, gst.MESSAGE_REQUEST_STATE)
            self.assertEquals(m.parse_request_state(), gst.STATE_NULL)

    def testBufferingStatsMessage(self):
        if hasattr(gst.Message, 'set_buffering_stats'):
            gst.debug("Creating buffering message")
            m = gst.message_new_buffering(self.element, 50)
            gst.debug("Setting stats")
            m.set_buffering_stats(gst.BUFFERING_LIVE, 30, 1024, 123456)
            self.assertEquals(m.type, gst.MESSAGE_BUFFERING)
            mode, ain, aout, left = m.parse_buffering_stats()
            self.assertEquals(mode, gst.BUFFERING_LIVE)
            self.assertEquals(ain, 30)
            self.assertEquals(aout, 1024)
            self.assertEquals(left, 123456)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import sys

import gi
gi.require_version('Gst', '1.0')
gi.require_version('GLib', '2.0')
from gi.repository import GLib, Gst


def bus_call(bus, message, loop):
    t = message.type
    if t == Gst.MessageType.EOS:
        sys.stdout.write("End-of-stream\n")
        loop.quit()
    elif t == Gst.MessageType.ERROR:
        err, debug = message.parse_error()
        sys.stderr.write("Error: %s: %s\n" % (err, debug))
        loop.quit()
    return True


def main(args):
    if len(args) != 2:
        sys.stderr.write("usage: %s <media file or uri>\n" % args[0])
        sys.exit(1)

    Gst.init(None)

    playbin = Gst.ElementFactory.make("playbin", None)
    if not playbin:
        sys.stderr.write("'playbin' gstreamer plugin missing\n")
        sys.exit(1)

    # take the commandline argument and ensure that it is a uri
    if Gst.uri_is_valid(args[1]):
        uri = args[1]
    else:
        uri = Gst.filename_to_uri(args[1])
    playbin.set_property('uri', uri)

    # create and event loop and feed gstreamer bus mesages to it
    loop = GLib.MainLoop()

    bus = playbin.get_bus()
    bus.add_signal_watch()
    bus.connect("message", bus_call, loop)

    # start play back and listed to events
    playbin.set_state(Gst.State.PLAYING)
    try:
        loop.run()
    except e:
        pass

    # cleanup
    playbin.set_state(Gst.State.NULL)


if __name__ == '__main__':
    sys.exit(main(sys.argv))



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Copyright (C) 2009 Thomas Vander Stichele
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
import overrides_hack
import sys
from common import TestCase, unittest
from gi.repository import Gst
import gi


gi.require_version('Gst', '1.0')


overrides_hack


class TestNotInitialized(TestCase):
    def testNotInitialized(self):
        if sys.version_info >= (3, 0):
            assert_type = Gst.NotInitialized
        else:
            assert_type = TypeError

        with self.assertRaises(assert_type):
            Gst.Caps.from_string("audio/x-raw")

        with self.assertRaises(assert_type):
            Gst.Structure.from_string("audio/x-raw")

        with self.assertRaises(assert_type):
            Gst.ElementFactory.make("identity", None)

    def testNotDeinitialized(self):
        Gst.init(None)

        self.assertIsNotNone(Gst.Caps.from_string("audio/x-raw"))
        self.assertIsNotNone(Gst.Structure.from_string("audio/x-raw"))
        self.assertIsNotNone(Gst.ElementFactory.make("identity", None))

        Gst.deinit()
        if sys.version_info >= (3, 0):
            assert_type = Gst.NotInitialized
        else:
            assert_type = TypeError

        with self.assertRaises(assert_type):
            Gst.Caps.from_string("audio/x-raw")

        with self.assertRaises(assert_type):
            Gst.Structure.from_string("audio/x-raw")

        with self.assertRaises(assert_type):
            Gst.ElementFactory.make("identity", None)



import gobject
gobject.threads_init()
import gtk
gtk.gdk.threads_init()
import hildon
import gst
import sys

# VideoWidget taken from play.py in gst-python examples
class VideoWidget(gtk.DrawingArea):
    def __init__(self):
        gtk.DrawingArea.__init__(self)
        self.imagesink = None
        self.unset_flags(gtk.DOUBLE_BUFFERED)

    def do_expose_event(self, event):
        if self.imagesink:
            self.imagesink.expose()
            return False
        else:
            return True

    def set_sink(self, sink):
        assert self.window.xid
        self.imagesink = sink
        self.imagesink.set_xwindow_id(self.window.xid)

class MaemoGstView:

    def __init__(self):
        # hildon has one program instance per app, so get instance
        self.p = hildon.Program.get_instance()
        # set name of application: this shows in titlebar
        gtk.set_application_name("Maemo GStreamer VideoTest")
        # stackable window in case we want more windows in future in app
        self.w = hildon.StackableWindow()
        box = gtk.VBox()
        self.video_widget = VideoWidget()
        # video widget we want to expand to size
        box.pack_start(self.video_widget, True, True, 0)
        # a button finger height to play/pause 
        self.button = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
            hildon.BUTTON_ARRANGEMENT_VERTICAL, title="Pause")
        self.button.connect_after("clicked", self.on_button_clicked)
        # don't want button to expand or fill, just stay finger height
        box.pack_start(self.button, False, False, 0)
        self.w.add(box)
        self.w.connect("delete-event", gtk.main_quit)
        self.p.add_window(self.w)
        self.w.show_all()
        self.start_streaming()

    def start_streaming(self):
        # we use ximagesink solely for screenshotting ability
        # less cpu usage would happen with videotestsrc ! xvimagesink
        self.pipeline = \
            gst.parse_launch("videotestsrc ! videoscale ! ximagesink")
        bus = self.pipeline.get_bus()
        # need to connect to sync message handler so we get the sink to be
        # embedded at the right time and not have a temporary new window
        bus.enable_sync_message_emission()
        bus.add_signal_watch()
        bus.connect("sync-message::element", self.on_sync_message)
        bus.connect("message", self.on_message)
        self.pipeline.set_state(gst.STATE_PLAYING)

    def on_sync_message(self, bus, message):
        if message.structure is None:
            return
        if message.structure.get_name() == 'prepare-xwindow-id':
            # all this is needed to sync with the X server before giving the
            # x id to the sink
            gtk.gdk.threads_enter()
            gtk.gdk.display_get_default().sync()
            self.video_widget.set_sink(message.src)
            message.src.set_property("force-aspect-ratio", True)
            gtk.gdk.threads_leave()

    def on_message(self, bus, message):
        if message.type == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            hildon.hildon_banner_show_information(self.w, '', 
                "Error: %s" % err)

    def on_button_clicked(self, widget):
        success, state, pending = self.pipeline.get_state(1)
        # do not listen if in middle of state change
        if not pending:
            if state == gst.STATE_PLAYING:
                self.pipeline.set_state(gst.STATE_PAUSED)
                self.button.set_label("Play")
            else:
                self.pipeline.set_state(gst.STATE_PLAYING)
                self.button.set_label("Pause")

def main():
    view = MaemoGstView()
    gtk.main()

if __name__ == '__main__':
    sys.exit(main())



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2007 Johan Dahlin
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

from gi.repository import Gst
import gi
from common import TestCase, unittest
import overrides_hack
overrides_hack


gi.require_version("Gst", "1.0")


class TestPlugin(TestCase):
    def testLoad(self):
        Gst.init(None)
        p = Gst.parse_launch("fakesrc ! test_identity_py name=id ! fakesink")
        assert p.get_by_name("id").transformed is False
        p.set_state(Gst.State.PLAYING)
        p.get_state(Gst.CLOCK_TIME_NONE)
        p.set_state(Gst.State.NULL)
        assert p.get_by_name("id").transformed is True

        del p


if __name__ == "__main__":
    unittest.main()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Copyright (C) 2005 Johan Dahlin
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import unittest
from common import gst, TestCase

class IteratorTest(TestCase):
    # XXX: Elements
    def testBinIterateElements(self):
        pipeline = gst.parse_launch("fakesrc name=src ! fakesink name=sink")
        elements = list(pipeline.elements())
        fakesrc = pipeline.get_by_name("src")
        fakesink = pipeline.get_by_name("sink")
        
        self.assertEqual(len(elements), 2)
        self.failUnless(fakesrc in elements)
        self.failUnless(fakesink in elements)

        pipeline.remove(fakesrc)
        elements = list(pipeline.elements())

        self.assertEqual(len(elements), 1)
        self.failUnless(not fakesrc in pipeline)

        # XXX : There seems to be a problem about the GType
        #       set in gst_bin_iterated_sorted

    def testBinIterateSorted(self):
        pipeline =  gst.parse_launch("fakesrc name=src ! fakesink name=sink")
        elements = list(pipeline.sorted())
        fakesrc = pipeline.get_by_name("src")
        fakesink = pipeline.get_by_name("sink")

        self.assertEqual(elements[0], fakesink)
        self.assertEqual(elements[1], fakesrc)

    def testBinIterateRecurse(self):
        pipeline = gst.parse_launch("fakesrc name=src ! fakesink name=sink")
        elements = list(pipeline.recurse())
        fakesrc = pipeline.get_by_name("src")
        fakesink = pipeline.get_by_name("sink")

        self.assertEqual(elements[0], fakesink)
        self.assertEqual(elements[1], fakesrc)

    def testBinIterateSinks(self):
        pipeline = gst.parse_launch("fakesrc name=src ! fakesink name=sink")
        elements = list(pipeline.sinks())
        fakesrc = pipeline.get_by_name("src")
        fakesink = pipeline.get_by_name("sink")

        self.assertEqual(len(elements), 1)
        self.failUnless(fakesink in elements)
        self.failUnless(not fakesrc in elements)

    
    def testIteratePadsFakeSrc(self):
        fakesrc = gst.element_factory_make('fakesrc')
        pads = list(fakesrc.pads())
        srcpad = fakesrc.get_pad('src')
        self.assertEqual(len(pads), 1)
        self.assertEqual(pads[0], srcpad)
        srcpads = list(fakesrc.src_pads())
        self.assertEqual(len(srcpads), 1)
        self.assertEqual(srcpads[0], srcpad)
        sinkpads = list(fakesrc.sink_pads())
        self.assertEqual(sinkpads, [])

        self.assertEqual(len(list(fakesrc)), 1)
        for pad in fakesrc:
            self.assertEqual(pad, srcpad)
            break
        else:
            raise AssertionError
        
    def testIteratePadsFakeSink(self):
        fakesink = gst.element_factory_make('fakesink')
        pads = list(fakesink.pads())
        sinkpad = fakesink.get_pad('sink')
        self.assertEqual(len(pads), 1)
        self.assertEqual(pads[0], sinkpad)
        srcpads = list(fakesink.src_pads())
        self.assertEqual(srcpads, [])
        sinkpads = list(fakesink.sink_pads())
        self.assertEqual(len(sinkpads), 1)
        self.assertEqual(sinkpads[0], sinkpad)

        self.assertEqual(len(list(fakesink)), 1)
        for pad in fakesink:
            self.assertEqual(pad, sinkpad)
            break
        else:
            raise AssertionError

    def testInvalidIterator(self):
        p = gst.Pad("p", gst.PAD_SRC)
        # The C function will return NULL, we should
        # therefore have an exception raised
        self.assertRaises(TypeError, p.iterate_internal_links)
        del p




# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2007 Johan Dahlin
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

from common import gst, TestCase


class TestTagList(TestCase):
    def testContains(self):
        taglist = gst.TagList()
        self.failIf('key' in taglist)
        taglist['key'] = 'value'
        self.failUnless('key' in taglist)

    def testLength(self):
        taglist = gst.TagList()
        self.assertEqual(len(taglist), 0)
        taglist['key1'] = 'value'
        taglist['key2'] = 'value'
        self.assertEqual(len(taglist), 2)

    def testKeys(self):
        taglist = gst.TagList()
        self.assertEqual(taglist.keys(), [])
        taglist['key1'] = 'value'
        taglist['key2'] = 'value'
        keys = taglist.keys()
        keys.sort()
        self.assertEqual(keys, ['key1', 'key2'])

    def testUnicode(self):
        taglist = gst.TagList()

        # normal ASCII text
        taglist[gst.TAG_ARTIST] = 'Artist'
        self.failUnless(isinstance(taglist[gst.TAG_ARTIST], unicode))
        self.assertEquals(taglist[gst.TAG_ARTIST], u'Artist')
        self.assertEquals(taglist[gst.TAG_ARTIST], 'Artist')

        # normal ASCII text as unicode
        taglist[gst.TAG_ARTIST] = u'Artist'
        self.failUnless(isinstance(taglist[gst.TAG_ARTIST], unicode))
        self.assertEquals(taglist[gst.TAG_ARTIST], u'Artist')
        self.assertEquals(taglist[gst.TAG_ARTIST], 'Artist')
        
        # real unicode
        taglist[gst.TAG_ARTIST] = u'S\xc3\xadgur R\xc3\xb3s'
        self.failUnless(isinstance(taglist[gst.TAG_ARTIST], unicode))
        self.assertEquals(taglist[gst.TAG_ARTIST], u'S\xc3\xadgur R\xc3\xb3s')
        
    def testUnsignedInt(self):
        taglist = gst.TagList()
        taglist[gst.TAG_TRACK_NUMBER] = 1
        vorbis = gst.tag.to_vorbis_comments(taglist, gst.TAG_TRACK_NUMBER)
        self.assertEquals(vorbis, ['TRACKNUMBER=1'])



#!/usr/bin/env python3
# Demonstration of using compositor and the samples-selected
# signal to do frame-by-frame updates and animation by
# udpating compositor pad properties and the GstVideoConverter
# config.
#
# Supply a URI argument to use a video file in the example,
# or omit it to just animate a videotestsrc.
import gi
import math
import sys

gi.require_version('Gst', '1.0')
gi.require_version('GstVideo', '1.0')
gi.require_version('GLib', '2.0')
gi.require_version('GObject', '2.0')
from gi.repository import GLib, GObject, Gst, GstVideo


def bus_call(bus, message, loop):
    t = message.type
    if t == Gst.MessageType.EOS:
        sys.stdout.write("End-of-stream\n")
        loop.quit()
    elif t == Gst.MessageType.ERROR:
        err, debug = message.parse_error()
        sys.stderr.write("Error: %s: %s\n" % (err, debug))
        loop.quit()
    return True


def update_compositor(comp, seg, pts, dts, duration, info):
    sink = comp.get_static_pad("sink_1")
    # Can peek at the input sample(s) here to get the real video input caps
    sample = comp.peek_next_sample(sink)
    caps = sample.get_caps()
    in_w = caps[0]['width']
    in_h = caps[0]['height']

    dest_w = 160 + in_w * math.fabs(math.sin(pts / (10 * Gst.SECOND) * math.pi))
    dest_h = 120 + in_h * math.fabs(math.sin(pts / (11 * Gst.SECOND) * math.pi))
    x = (1920 - dest_w) * math.fabs(math.sin(pts / (5 * Gst.SECOND) * math.pi))
    y = (1080 - dest_h) * math.fabs(math.sin(pts / (7 * Gst.SECOND) * math.pi))

    sink.set_property("xpos", x)
    sink.set_property("ypos", y)
    sink.set_property("width", dest_w)
    sink.set_property("height", dest_h)

    # Update video-converter settings
    cfg = Gst.Structure.new_empty("GstVideoConverter")

    # When scaling down, switch to nearest-neighbour scaling, otherwise use bilinear
    if in_w < dest_w or in_h < dest_h:
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_RESAMPLER_METHOD, GstVideo.VideoResamplerMethod.NEAREST)
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_CHROMA_RESAMPLER_METHOD, GstVideo.VideoResamplerMethod.NEAREST)
    else:
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_RESAMPLER_METHOD, GstVideo.VideoResamplerMethod.LINEAR)
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_CHROMA_RESAMPLER_METHOD, GstVideo.VideoResamplerMethod.LINEAR)

    # Crop some from the top and bottom or sides of the source image
    crop = 64 * math.cos(pts / (4 * Gst.SECOND) * math.pi)
    if crop < 0:
        crop = min(in_w / 2, -crop)
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_SRC_X, int(crop))
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_SRC_WIDTH, int(in_w - 2 * crop))
    else:
        crop = min(in_h / 2, crop)
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_SRC_Y, int(crop))
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_SRC_HEIGHT, int(in_h - 2 * crop))

    # Add add some borders to the result by not filling the destination rect
    border = 64 * math.sin(pts / (4 * Gst.SECOND) * math.pi)
    if border < 0:
        border = min(in_w / 2, -border)
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_DEST_X, int(border))
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_DEST_WIDTH, int(dest_w - 2 * border))
    else:
        border = min(in_h / 2, border)
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_DEST_Y, int(border))
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_DEST_HEIGHT, int(dest_h - 2 * border))

    # Set the border colour. Need to set this into a GValue explicitly to ensure it's a uint
    argb = GObject.Value()
    argb.init(GObject.TYPE_UINT)
    argb.set_uint(0x80FF80FF)
    cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_BORDER_ARGB, argb)
    cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_FILL_BORDER, True)

    # When things get wide, do some dithering why not
    if dest_w > in_w:
        # Dither quantization must also be a uint
        dither = GObject.Value()
        dither.init(GObject.TYPE_UINT)
        dither.set_uint(64)

        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_DITHER_METHOD, GstVideo.VideoDitherMethod.FLOYD_STEINBERG)
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_DITHER_QUANTIZATION, dither)
    else:
        dither = GObject.Value()
        dither.init(GObject.TYPE_UINT)
        dither.set_uint(1)

        # cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_DITHER_METHOD, GstVideo.VideoDitherMethod.NONE)
        cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_DITHER_QUANTIZATION, dither)

    # and fade it in and out every 13 seconds
    alpha = 0.25 + 0.75 * math.fabs(math.sin(pts / (13 * Gst.SECOND) * math.pi))
    # Use the video-converter to render alpha
    # cfg.set_value(GstVideo.VIDEO_CONVERTER_OPT_ALPHA_VALUE, alpha)
    # or use the pad's alpha setting is better because compositor can use it to skip rendering
    sink.set_property("alpha", alpha)

    # print(f"Setting config {cfg}")
    sink.set_property("converter-config", cfg)


if __name__ == "__main__":
    Gst.init(sys.argv)

    pipeline = Gst.ElementFactory.make('pipeline', None)

    compositor = Gst.ElementFactory.make("compositor", None)
    compositor.set_property("emit-signals", True)
    compositor.set_property("background", "black")
    compositor.connect("samples-selected", update_compositor)

    cf = Gst.ElementFactory.make("capsfilter", None)
    # Need to make sure the output has RGBA or AYUV, or we can't do alpha blending / fades:
    caps = Gst.Caps.from_string("video/x-raw,width=1920,height=1080,framerate=25/1,format=RGBA")
    cf.set_property("caps", caps)

    conv = Gst.ElementFactory.make("videoconvert", None)
    sink = Gst.ElementFactory.make("autovideosink", None)

    pipeline.add(compositor, cf, conv, sink)
    Gst.Element.link_many(compositor, cf, conv, sink)

    bgsource = Gst.parse_bin_from_description("videotestsrc pattern=circular ! capsfilter name=cf", False)

    cfsrc = bgsource.get_by_name("cf")
    caps = Gst.Caps.from_string("video/x-raw,width=320,height=180,framerate=1/1,format=RGB")
    cfsrc.set_property("caps", caps)
    src = cfsrc.get_static_pad("src")
    bgsource.add_pad(Gst.GhostPad.new("src", src))

    pipeline.add(bgsource)
    bgsource.link(compositor)

    pad = compositor.get_static_pad("sink_0")
    pad.set_property("width", 1920)
    pad.set_property("height", 1080)

    if len(sys.argv) > 1:
        source = Gst.parse_bin_from_description("uridecodebin name=u ! capsfilter name=cf caps=video/x-raw", False)
        u = source.get_by_name("u")
        u.set_property("uri", sys.argv[1])

        cfsrc = source.get_by_name("cf")
        caps = Gst.Caps.from_string("video/x-raw")
        cfsrc.set_property("caps", caps)

        # Expose the capsfilter source pad as a ghost pad
        src = cfsrc.get_static_pad("src")
        source.add_pad(Gst.GhostPad.new("src", src))
    else:
        source = Gst.parse_bin_from_description("videotestsrc ! capsfilter name=cf", False)
        cfsrc = source.get_by_name("cf")
        caps = Gst.Caps.from_string("video/x-raw,width=320,height=240,framerate=30/1,format=I420")
        cfsrc.set_property("caps", caps)

        src = cfsrc.get_static_pad("src")
        source.add_pad(Gst.GhostPad.new("src", src))

    pipeline.add(source)
    source.link(compositor)

    pipeline.set_state(Gst.State.PLAYING)

    bus = pipeline.get_bus()
    bus.add_signal_watch()

    loop = GLib.MainLoop()
    bus.connect("message", bus_call, loop)
    loop.run()

    pipeline.set_state(Gst.State.NULL)
    pipeline.get_state(Gst.CLOCK_TIME_NONE)



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# audio-controller.py
# (c) 2005 Edward Hervey <edward at fluendo dot com>
# Test case for the GstController on sinesrc -> alsasink
# Inspired from ensonic's examples/controller/audio-controller.c

import pygst
pygst.require('0.10')
import gst
import time

def main():
    pipeline = gst.Pipeline("audiocontroller")
    src = gst.element_factory_make("audiotestsrc", "src")
    sink = gst.element_factory_make("alsasink", "sink")
    pipeline.add(src, sink)
    src.link(sink)

    control = gst.Controller(src, "freq", "volume")
    control.set_interpolation_mode("volume", gst.INTERPOLATE_LINEAR)
    control.set_interpolation_mode("freq", gst.INTERPOLATE_LINEAR)

    control.set("volume", 0, 0.0)
    control.set("volume", 2 * gst.SECOND, 1.0)
    control.set("volume", 4 * gst.SECOND, 0.0)
    control.set("volume", 6 * gst.SECOND, 1.0)

    control.set("freq", 0, 440.0)
    control.set("freq", 3 * gst.SECOND, 3000.0)
    control.set("freq", 6 * gst.SECOND, 880.0)

    pipeline.set_state(gst.STATE_PLAYING)

    time.sleep(7)

if __name__ == "__main__":
    main()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Copyright (C) 2009 Thomas Vander Stichele
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
import overrides_hack
import sys
from common import TestCase, unittest
from gi.repository import Gst
import gi


gi.require_version('Gst', '1.0')


overrides_hack


class TimeArgsTest(TestCase):
    def testNoneTime(self):
        self.assertRaises(TypeError, Gst.TIME_ARGS, None)

    def testStringTime(self):
        self.assertRaises(TypeError, Gst.TIME_ARGS, "String")

    def testClockTimeNone(self):
        self.assertEqual(Gst.TIME_ARGS(Gst.CLOCK_TIME_NONE), 'CLOCK_TIME_NONE')

    def testOneSecond(self):
        self.assertEqual(Gst.TIME_ARGS(Gst.SECOND), '0:00:01.000000000')


class TestCaps(TestCase):

    def test_writable_make_writable_no_copy(self):
        Gst.init(None)
        caps = Gst.Caps("audio/x-raw")
        repr = caps.__repr__()
        caps.make_writable()
        self.assertEqual(repr, caps.__repr__())
        del caps

    def test_make_writable_with_copy(self):
        Gst.init(None)

        caps = Gst.Caps("audio/x-raw")
        repr = caps.__repr__()
        miniobj = caps.mini_object
        self.assertFalse(caps.is_writable())
        repr = caps.__repr__()
        self.assertTrue(caps.make_writable())
        self.assertNotEqual(repr, caps.__repr__())
        self.assertTrue(caps.is_writable())
        del caps
        del miniobj

    def test_no_writable(self):
        Gst.init(None)
        caps = Gst.Caps("audio/x-raw")
        caps.mini_object.refcount += 1

        with self.assertRaises(Gst.NotWritableCaps):
            with caps.writable_structure(0) as s:
                s.set_value("rate", 44100)

        caps.mini_object.refcount -= 1

    def test_make_writable(self):
        Gst.init(None)
        caps = Gst.Caps("audio/x-raw")

        capsfilter = Gst.ElementFactory.make("capsfilter", None)

        if not capsfilter:
            self.skipTest("capsfilter not available")

        capsfilter.set_property("caps", caps)
        caps = capsfilter.get_property("caps")

        with self.assertRaises(Gst.NotWritableCaps):
            with caps.writable_structure(0) as s:
                s.set_value("rate", 44100)
        caps.make_writable()
        with caps.writable_structure(0) as s:
            s.set_value("rate", 44100)
        capsfilter.set_property("caps", caps)
        caps = capsfilter.get_property("caps")
        with caps.get_structure(0) as s:
            self.assertEqual(s["rate"], 44100)

    def test_writable(self):
        Gst.init(None)
        caps = Gst.Caps("audio/x-raw")

        with caps.writable_structure(0) as s:
            s.set_value("rate", 44100)
            s.set_value("channels", 2)
        with caps.get_structure(0) as s:
            self.assertEqual(s["rate"], 44100)
            self.assertEqual(s["channels"], 2)

    def test_delete_caps_while_accessing(self):
        Gst.init(None)
        caps = Gst.Caps("audio/x-raw")

        with caps.writable_structure(0) as s:
            del caps
            s.set_value("rate", 44100)
            s.set_value("channels", 2)
            self.assertEqual(s["rate"], 44100)
            self.assertEqual(s["channels"], 2)

    def test_read_no_copy(self):
        Gst.init(None)
        caps = Gst.Caps("audio/x-raw")

        with caps.get_structure(0) as s:
            ptr = s.__ptr__()

        with caps.get_structure(0) as s:
            self.assertEqual(ptr, s.__ptr__())

        c2 = caps.mini_object
        self.assertEqual(c2.refcount, 2)
        with caps.get_structure(0) as s:
            with self.assertRaises(Gst.NotWritableStructure):
                s.set_value("rate", 44100)
        caps.make_writable()

        with caps.get_structure(0) as s:
            self.assertNotEqual(ptr, s.__ptr__())

    def test_read_no_copy_no_cm(self):
        Gst.init(None)
        caps = Gst.Caps("audio/x-raw")

        s = caps.get_structure(0)
        ptr = s.__ptr__()

        s2 = caps.get_structure(0)
        self.assertEqual(ptr, s2.__ptr__())

        c2 = caps.mini_object
        self.assertEqual(c2.refcount, 2)
        with self.assertRaises(Gst.NotWritableStructure):
            caps.get_structure(0).set_value("rate", 44100)

        caps.make_writable()
        s = caps.get_structure(0)
        self.assertNotEqual(ptr, s.__ptr__())

    def test_iterate(self):
        Gst.init(None)
        caps = Gst.Caps()
        caps.append_structure(Gst.Structure('test1', one=1))
        caps.append_structure(Gst.Structure('test2', two=2))
        caps.append_structure(Gst.Structure('test3', three=3))
        self.assertEqual(len(caps), 3)
        items = [s for s in caps]
        self.assertEqual(items[0]['one'], 1)
        self.assertEqual(items[1]['two'], 2)
        self.assertEqual(items[2]['three'], 3)
        with self.assertRaises(IndexError):
            caps[3]


class TestStructure(TestCase):

    def test_new(self):
        Gst.init(None)
        test = Gst.Structure('test', test=1)
        self.assertEqual(test['test'], 1)

        test = Gst.Structure('test,test=1')
        self.assertEqual(test['test'], 1)

    def test_iterate_items(self):
        Gst.init(None)
        test = Gst.Structure('test', one=1, two=2, three=3)
        self.assertEqual(len(test), 3)
        self.assertEqual(test["one"], 1)
        self.assertEqual(test["two"], 2)
        self.assertEqual(test["three"], 3)
        items = {k: v for k, v in test.items()}
        self.assertEqual(items, {'one': 1, 'two': 2, 'three': 3})
        keys = [k for k in test]
        self.assertEqual(keys, ['one', 'two', 'three'])
        with self.assertRaises(KeyError):
            test["four"]


class TestEvent(TestCase):
    def test_writable(self):
        Gst.init(None)
        event = Gst.Event.new_caps(Gst.Caps("audio/x-raw"))

        with event.writable_structure() as s:
            s.set_value("rate", 44100)
            s.set_value("channels", 2)

        with event.get_structure() as s:
            self.assertEqual(s["rate"], 44100)
            self.assertEqual(s["channels"], 2)

    def test_no_structure(self):
        Gst.init(None)
        event = Gst.Event.new_flush_start()
        s = event.get_structure()
        self.assertIsNone(s)


class TestQuery(TestCase):
    def test_writable(self):
        Gst.init(None)
        query = Gst.Query.new_caps(Gst.Caps("audio/x-raw"))

        with query.writable_structure() as s:
            del query
            s.set_value("rate", 44100)
            s.set_value("channels", 2)
            self.assertEqual(s["rate"], 44100)
            self.assertEqual(s["channels"], 2)


class TestContext(TestCase):
    def test_writable(self):
        Gst.init(None)
        context = Gst.Context.new("test", True)

        with context.writable_structure() as s:
            s.set_value("one", 1)
            s.set_value("two", 2)
            self.assertEqual(s["one"], 1)
            self.assertEqual(s["two"], 2)


class TestBin(TestCase):

    def test_add_pad(self):
        Gst.init(None)
        self.assertEqual(Gst.ElementFactory.make("bin", None).sinkpads, [])

    def test_make_and_add(self):
        Gst.init(None)
        bin = Gst.Bin()
        with self.assertRaises(Gst.MissingPluginError):
            bin.make_and_add("nonexistent")
        e = bin.make_and_add("identity")
        # Adding the same element again should fail
        with self.assertRaises(Gst.AddError):
            bin.add(e)

    def test_iterate(self):
        Gst.init(None)
        bin = Gst.Bin()
        e = bin.make_and_add("identity")
        chidren = [c for c in bin]
        self.assertEqual(chidren, [e])


class TestBuffer(TestCase):

    def test_set_metas(self):
        Gst.init(None)
        buf = Gst.Buffer.new_wrapped([42])
        buf.pts = 42
        self.assertEqual(buf.pts, 42)

        make_not_writable = buf.mini_object
        with self.assertRaises(Gst.NotWritableMiniObject):
            buf.pts = 52

        buf.make_writable()
        buf.dts = 62
        self.assertEqual(buf.dts, 62)
        self.assertTrue(buf.flags & Gst.BufferFlags.DISCONT == 0)
        buf.flags = Gst.BufferFlags.DISCONT
        buf.duration = 72
        self.assertEqual(buf.duration, 72)
        buf.offset = 82
        self.assertEqual(buf.offset, 82)
        buf.offset_end = 92
        self.assertEqual(buf.offset_end, 92)
        del make_not_writable

        meta = buf.add_reference_timestamp_meta(Gst.Caps("yes"), 10, 10)
        self.assertEqual(buf.get_reference_timestamp_meta(), meta)

    def test_map_unmap_manual(self):
        Gst.init(None)
        buf = Gst.Buffer.new_wrapped([42])
        info = buf.map(Gst.MapFlags.READ | Gst.MapFlags.WRITE)
        self.assertEqual(info.data[0], 42)
        buf.unmap(info)
        with self.assertRaises(ValueError):
            info.data[0]

    def test_map_unmap_context(self):
        Gst.init(None)
        buf = Gst.Buffer.new_wrapped([42])
        with buf.map(Gst.MapFlags.READ | Gst.MapFlags.WRITE) as info:
            self.assertEqual(info.data[0], 42)
        with self.assertRaises(ValueError):
            info.data[0]


class TestPadProbe(TestCase):

    def test_pad_probe(self):
        Gst.init(None)
        pipeline = Gst.Pipeline("pipeline")
        src = pipeline.make_and_add("fakesrc", "src")
        sink = pipeline.make_and_add("fakesink", "sink")
        src.link(sink)

        src.props.num_buffers = 5

        buffer_count = 0

        def probe_cb(pad, info):
            nonlocal buffer_count

            buffer_count += 1

            # Get a writable buffer from the probe info
            with info.writable_object() as buffer:
                self.assertIsInstance(buffer, Gst.Buffer)
                self.assertTrue(buffer.is_writable())
                buffer.pts = buffer_count * Gst.SECOND * 2
                ptr = buffer.__ptr__()
                # Info does not hold a buffer any more
                self.assertIsNone(info.get_buffer())

            # info.get_buffer() never returns a writable buffer because both
            # python and GstPadProbeInfo hold references to it.
            buffer = info.get_buffer()
            self.assertIsNotNone(buffer)
            self.assertFalse(buffer.is_writable())
            self.assertEqual(buffer.pts, buffer_count * Gst.SECOND * 2)
            self.assertEqual(buffer.__ptr__(), ptr)

            return Gst.PadProbeReturn.OK

        sink_pad = sink.get_static_pad("sink")
        probe_id = sink_pad.add_probe(Gst.PadProbeType.BUFFER, probe_cb)

        pipeline.set_state(Gst.State.PLAYING)
        bus = pipeline.get_bus()
        msg = bus.timed_pop_filtered(Gst.SECOND * 5, Gst.MessageType.EOS)
        self.assertIsNotNone(msg)

        # We should have seen exactly 5 buffers
        self.assertEqual(buffer_count, 5)

        # Check that last buffer has the expected PTS
        sample = sink.props.last_sample
        buffer = sample.get_buffer()
        self.assertEqual(buffer.pts, buffer_count * Gst.SECOND * 2)

        sink_pad.remove_probe(probe_id)
        pipeline.set_state(Gst.State.NULL)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# GStreamer python bindings
# Copyright (C) 2002 David I. Lehn <dlehn@users.sourceforge.net>
#               2004 Johan Dahlin  <johan@gnome.org>
#
# filesrc.py: implements a file source element completely in python
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
import gi
import sys

gi.require_version('Gst', '1.0')
gi.require_version('GstBase', '1.0')
from gi.repository import GLib, Gst, GObject, GstBase


if __name__ == '__main__':
    Gst.init(None)
Gst.init_python()


class FileSource(GstBase.BaseSrc):
    __gstmetadata__ = ('PyFileSource', 'Source',
                       'Custom Python filesrc example', 'Jan Schmidt')

    __gsttemplates__ = (
        Gst.PadTemplate.new("src",
                            Gst.PadDirection.SRC,
                            Gst.PadPresence.ALWAYS,
                            Gst.Caps.new_any()),
    )

    blocksize = 4096
    fd = None

    def __init__(self, name):
        super().__init__()
        self.curoffset = 0
        self.set_name(name)

    def set_property(self, name, value):
        if name == 'location':
            self.fd = open(value, 'rb')

    def do_create(self, offset, size, wot):
        if offset != self.curoffset:
            self.fd.seek(offset, 0)
        data = self.fd.read(self.blocksize)
        if data:
            self.curoffset += len(data)
            return Gst.FlowReturn.OK, Gst.Buffer.new_wrapped(data)
        else:
            return Gst.FlowReturn.EOS, None


GObject.type_register(FileSource)
__gstelementfactory__ = ("filesrc_py", Gst.Rank.NONE, FileSource)


def main(args):
    Gst.init()

    if len(args) != 3:
        print(f'Usage: {args[0]} input output')
        return -1

    bin = Gst.Pipeline('pipeline')

    filesrc = FileSource('filesource')
    assert filesrc
    filesrc.set_property('location', args[1])

    filesink = Gst.ElementFactory.make('filesink', 'sink')
    filesink.set_property('location', args[2])

    bin.add(filesrc, filesink)
    Gst.Element.link_many(filesrc, filesink)

    bin.set_state(Gst.State.PLAYING)
    mainloop = GLib.MainLoop()

    def bus_event(bus, message):
        t = message.type
        if t == Gst.MessageType.EOS:
            mainloop.quit()
        elif t == Gst.MessageType.ERROR:
            err, debug = message.parse_error()
            print(f"Error: {err}, {debug}")
            mainloop.quit()
        return True
    bus = bin.get_bus()
    bus.add_signal_watch()
    bus.connect('message', bus_event)

    mainloop.run()
    bin.set_state(Gst.State.NULL)


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/env python

import sys
import traceback
from math import pi

import pygtk
pygtk.require ("2.0")
import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')
import gst

import cairo

WIDTH, HEIGHT = 640, 480
FRAMES = 300
FRAMERATE = 15

class PyGstBufferDraw(gst.Element):
    _sinkpadtemplate = gst.PadTemplate ("sink",
                                         gst.PAD_SINK,
                                         gst.PAD_ALWAYS,
                                         gst.caps_from_string ("video/x-raw-rgb,bpp=32,depth=32,blue_mask=-16777216,green_mask=16711680, red_mask=65280, alpha_mask=255,width=[ 1, 2147483647 ],height=[ 1, 2147483647 ],framerate=[ 0/1, 2147483647/1 ]"))
    _srcpadtemplate = gst.PadTemplate ("src",
                                         gst.PAD_SRC,
                                         gst.PAD_ALWAYS,
                                         gst.caps_from_string ("video/x-raw-rgb,bpp=32,depth=32,blue_mask=-16777216,green_mask=16711680, red_mask=65280, alpha_mask=255,width=[ 1, 2147483647 ],height=[ 1, 2147483647 ],framerate=[ 0/1, 2147483647/1 ]"))

    def __init__(self):
        gst.Element.__init__(self)

        self.sinkpad = gst.Pad(self._sinkpadtemplate, "sink")
        self.sinkpad.set_chain_function(self.chainfunc)
        self.sinkpad.set_event_function(self.eventfunc)
        self.sinkpad.set_getcaps_function(gst.Pad.proxy_getcaps)
        self.sinkpad.set_setcaps_function(gst.Pad.proxy_setcaps)
        self.add_pad (self.sinkpad)

        self.srcpad = gst.Pad(self._srcpadtemplate, "src")

        self.srcpad.set_event_function(self.srceventfunc)
        self.srcpad.set_query_function(self.srcqueryfunc)
        self.srcpad.set_getcaps_function(gst.Pad.proxy_getcaps)
        self.srcpad.set_setcaps_function(gst.Pad.proxy_setcaps)
        self.add_pad (self.srcpad)

    def chainfunc(self, pad, buffer):
        try:
            outbuf = buffer.copy_on_write ()
            self.draw_on (outbuf)
            return self.srcpad.push (outbuf)
        except:
            return GST_FLOW_ERROR

    def eventfunc(self, pad, event):
        return self.srcpad.push_event (event)
        
    def srcqueryfunc (self, pad, query):
        return self.sinkpad.query (query)
    def srceventfunc (self, pad, event):
        return self.sinkpad.push_event (event)

    def draw_on (self, buf):
        try:
            caps = buf.get_caps()
            width = caps[0]['width']
            height = caps[0]['height']
            framerate = caps[0]['framerate']
            surface = cairo.ImageSurface.create_for_data (buf, cairo.FORMAT_ARGB32, width, height, 4 * width)
            ctx = cairo.Context(surface)
        except:
            print "Failed to create cairo surface for buffer"
            traceback.print_exc()
            return

        try:
            center_x = width/4
            center_y = 3*height/4

            # draw a circle
            radius = float (min (width, height)) * 0.25
            ctx.set_source_rgba (0.0, 0.0, 0.0, 0.9)
            ctx.move_to (center_x, center_y)
            ctx.arc (center_x, center_y, radius, 0, 2.0*pi)
            ctx.close_path()
            ctx.fill()
            ctx.set_source_rgba (1.0, 1.0, 1.0, 1.0)
            ctx.set_font_size(0.3 * radius)
            txt = "Hello World"
            extents = ctx.text_extents (txt)
            ctx.move_to(center_x - extents[2]/2, center_y + extents[3]/2)
            ctx.text_path(txt)
            ctx.fill()

        except:
            print "Failed cairo render"
            traceback.print_exc()

gobject.type_register(PyGstBufferDraw)

pipe = gst.Pipeline()
vt = gst.element_factory_make ("videotestsrc")
cf = gst.element_factory_make ("capsfilter")
c1 = PyGstBufferDraw()
color = gst.element_factory_make ("ffmpegcolorspace")
scale = gst.element_factory_make ("videoscale")
q1 = gst.element_factory_make ("queue")
sink = gst.element_factory_make ("autovideosink")

caps = gst.caps_from_string ("video/x-raw-rgb,width=%d,height=%d,framerate=%d/1" % (WIDTH, HEIGHT, FRAMERATE))
cf.set_property ("caps", caps)

vt.set_property ("num-buffers", FRAMES)

pipe.add (vt, cf, c1, q1, color, scale, sink)
gst.element_link_many (vt, cf, c1, q1, color, scale, sink)

def on_eos (bus, msg):
    mainloop.quit()

bus = pipe.get_bus()
bus.add_signal_watch()
bus.connect('message::eos', on_eos)

pipe.set_state (gst.STATE_PLAYING)

mainloop = gobject.MainLoop()
try:
    mainloop.run()
except:
    pass

pipe.set_state (gst.STATE_NULL)
pipe.get_state (gst.CLOCK_TIME_NONE)




#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# identity.py
# 2016 Marianna S. Buschle <msb@qtec.com>
#
# Simple identity element in python
#
# You can run the example from the source doing from gst-python/:
#
#  $ export GST_PLUGIN_PATH=$GST_PLUGIN_PATH:$PWD/plugin:$PWD/examples/plugins
#  $ GST_DEBUG=python:4 gst-launch-1.0 fakesrc num-buffers=10 ! identity_py ! fakesink

import gi
gi.require_version('GstBase', '1.0')

from gi.repository import Gst, GObject, GstBase
Gst.init_python()

#
# Simple Identity element created entirely in python
#
class Identity(GstBase.BaseTransform):
    __gstmetadata__ = ('Identity Python','Transform', \
                      'Simple identity element written in python', 'Marianna S. Buschle')

    __gsttemplates__ = (Gst.PadTemplate.new("src",
                                           Gst.PadDirection.SRC,
                                           Gst.PadPresence.ALWAYS,
                                           Gst.Caps.new_any()),
                       Gst.PadTemplate.new("sink",
                                           Gst.PadDirection.SINK,
                                           Gst.PadPresence.ALWAYS,
                                           Gst.Caps.new_any()))

    def do_transform_ip(self, buffer):
        Gst.info("timestamp(buffer):%s" % (Gst.TIME_ARGS(buffer.pts)))
        return Gst.FlowReturn.OK

GObject.type_register(Identity)
__gstelementfactory__ = ("identity_py", Gst.Rank.NONE, Identity)



#!/usr/bin/env python3

'''
Simple example to demonstrate using GstRtspServer in Python with video source from camera
'''
import argparse
import gi
gi.require_version("Gst", "1.0")
gi.require_version("GstRtspServer", "1.0")
from gi.repository import GLib, Gst, GstRtspServer


def start_rtsp_server(rtsp_port, device, stream_name):
    server = GstRtspServer.RTSPServer.new()
    server.props.service = rtsp_port
    server.attach(None)

    factory = GstRtspServer.RTSPMediaFactory.new()
    # Define pipeline which will be created during connection
    launch_str = f"( v4l2src device={device} ! videoconvert ! queue ! video/x-raw,format=I420 ! x264enc ! h264parse ! rtph264pay name=pay0 pt=96 )"
    factory.set_launch(launch_str)
    factory.set_shared(True)

    server.get_mount_points().add_factory("/" + stream_name, factory)
    print(f"Device: {device} \nUnder: rtsp://localhost:{rtsp_port}/{stream_name}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--camera",
        help="Camera file path",
        type=str,
        default="/dev/video0",
    )
    parser.add_argument("--rtsp-port", default="8557")
    parser.add_argument("--stream-name", default="cam")
    args = parser.parse_args()
    Gst.init(None)
    start_rtsp_server(args.rtsp_port, args.camera, args.stream_name)
    loop = GLib.MainLoop()
    loop.run()



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# gst-python
# Copyright (C) 2002 David I. Lehn
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
# 
# Author: David I. Lehn <dlehn@users.sourceforge.net>
#

import sys

import pygst
pygst.require('0.10')

import gst

def handoff_cb(sender, *args):
   print sender.get_name(), args

def main(args):
   # create a new bin to hold the elements
   #gst_debug_set_categories(-1)
   bin = gst.parse_launch('fakesrc name=source silent=1 num-buffers=10 signal-handoffs=true ! ' +
                          'fakesink name=sink silent=1 signal-handoffs=true')
   source = bin.get_by_name('source')
   source.connect('handoff', handoff_cb)
   source.get_pad("src").connect("have-data", handoff_cb)
   sink = bin.get_by_name('sink')
   sink.connect('handoff', handoff_cb)
   sink.get_pad("sink").connect('have-data', handoff_cb)

   print source, sink

   bus = bin.get_bus()
   
   res = bin.set_state(gst.STATE_PLAYING);
   assert res

   while 1:
      msg = bus.poll(gst.MESSAGE_EOS | gst.MESSAGE_ERROR, gst.SECOND)
      if msg:
         break

   res = bin.set_state(gst.STATE_NULL)
   assert res

if __name__ == '__main__':
   sys.exit(main(sys.argv))



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

import pygtk
pygtk.require('2.0')

import sys

import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')
import gst
import gst.interfaces
import gtk
gtk.gdk.threads_init()

class GstPlayer:
    def __init__(self, videowidget):
        self.playing = False
        self.player = gst.element_factory_make("playbin", "player")
        self.videowidget = videowidget
        self.on_eos = False

        bus = self.player.get_bus()
        bus.enable_sync_message_emission()
        bus.add_signal_watch()
        bus.connect('sync-message::element', self.on_sync_message)
        bus.connect('message', self.on_message)

    def on_sync_message(self, bus, message):
        if message.structure is None:
            return
        if message.structure.get_name() == 'prepare-xwindow-id':
            # Sync with the X server before giving the X-id to the sink
            gtk.gdk.threads_enter()
            gtk.gdk.display_get_default().sync()
            self.videowidget.set_sink(message.src)
            message.src.set_property('force-aspect-ratio', True)
            gtk.gdk.threads_leave()
            
    def on_message(self, bus, message):
        t = message.type
        if t == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            print "Error: %s" % err, debug
            if self.on_eos:
                self.on_eos()
            self.playing = False
        elif t == gst.MESSAGE_EOS:
            if self.on_eos:
                self.on_eos()
            self.playing = False

    def set_location(self, location):
        self.player.set_property('uri', location)

    def query_position(self):
        "Returns a (position, duration) tuple"
        try:
            position, format = self.player.query_position(gst.FORMAT_TIME)
        except:
            position = gst.CLOCK_TIME_NONE

        try:
            duration, format = self.player.query_duration(gst.FORMAT_TIME)
        except:
            duration = gst.CLOCK_TIME_NONE

        return (position, duration)

    def seek(self, location):
        """
        @param location: time to seek to, in nanoseconds
        """
        gst.debug("seeking to %r" % location)
        event = gst.event_new_seek(1.0, gst.FORMAT_TIME,
            gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
            gst.SEEK_TYPE_SET, location,
            gst.SEEK_TYPE_NONE, 0)

        res = self.player.send_event(event)
        if res:
            gst.info("setting new stream time to 0")
            self.player.set_new_stream_time(0L)
        else:
            gst.error("seek to %r failed" % location)

    def pause(self):
        gst.info("pausing player")
        self.player.set_state(gst.STATE_PAUSED)
        self.playing = False

    def play(self):
        gst.info("playing player")
        self.player.set_state(gst.STATE_PLAYING)
        self.playing = True
        
    def stop(self):
        self.player.set_state(gst.STATE_NULL)
        gst.info("stopped player")

    def get_state(self, timeout=1):
        return self.player.get_state(timeout=timeout)

    def is_playing(self):
        return self.playing
    
class VideoWidget(gtk.DrawingArea):
    def __init__(self):
        gtk.DrawingArea.__init__(self)
        self.imagesink = None
        self.unset_flags(gtk.DOUBLE_BUFFERED)

    def do_expose_event(self, event):
        if self.imagesink:
            self.imagesink.expose()
            return False
        else:
            return True

    def set_sink(self, sink):
        assert self.window.xid
        self.imagesink = sink
        self.imagesink.set_xwindow_id(self.window.xid)

class PlayerWindow(gtk.Window):
    UPDATE_INTERVAL = 500
    def __init__(self):
        gtk.Window.__init__(self)
        self.set_default_size(410, 325)

        self.create_ui()

        self.player = GstPlayer(self.videowidget)

        def on_eos():
            self.player.seek(0L)
            self.play_toggled()
        self.player.on_eos = lambda *x: on_eos()
        
        self.update_id = -1
        self.changed_id = -1
        self.seek_timeout_id = -1

        self.p_position = gst.CLOCK_TIME_NONE
        self.p_duration = gst.CLOCK_TIME_NONE

        def on_delete_event():
            self.player.stop()
            gtk.main_quit()
        self.connect('delete-event', lambda *x: on_delete_event())

    def load_file(self, location):
        self.player.set_location(location)
                                  
    def create_ui(self):
        vbox = gtk.VBox()
        self.add(vbox)

        self.videowidget = VideoWidget()
        vbox.pack_start(self.videowidget)
        
        hbox = gtk.HBox()
        vbox.pack_start(hbox, fill=False, expand=False)
        
        self.pause_image = gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE,
                                                    gtk.ICON_SIZE_BUTTON)
        self.pause_image.show()
        self.play_image = gtk.image_new_from_stock(gtk.STOCK_MEDIA_PLAY,
                                                   gtk.ICON_SIZE_BUTTON)
        self.play_image.show()
        self.button = button = gtk.Button()
        button.add(self.play_image)
        button.set_property('can-default', True)
        button.set_focus_on_click(False)
        button.show()
        hbox.pack_start(button, False)
        button.set_property('has-default', True)
        button.connect('clicked', lambda *args: self.play_toggled())
        
        self.adjustment = gtk.Adjustment(0.0, 0.00, 100.0, 0.1, 1.0, 1.0)
        hscale = gtk.HScale(self.adjustment)
        hscale.set_digits(2)
        hscale.set_update_policy(gtk.UPDATE_CONTINUOUS)
        hscale.connect('button-press-event', self.scale_button_press_cb)
        hscale.connect('button-release-event', self.scale_button_release_cb)
        hscale.connect('format-value', self.scale_format_value_cb)
        hbox.pack_start(hscale)
        self.hscale = hscale

        self.videowidget.connect_after('realize',
                                       lambda *x: self.play_toggled())

    def play_toggled(self):
        self.button.remove(self.button.child)
        if self.player.is_playing():
            self.player.pause()
            self.button.add(self.play_image)
        else:
            self.player.play()
            if self.update_id == -1:
                self.update_id = gobject.timeout_add(self.UPDATE_INTERVAL,
                                                     self.update_scale_cb)
            self.button.add(self.pause_image)

    def scale_format_value_cb(self, scale, value):
        if self.p_duration == -1:
            real = 0
        else:
            real = value * self.p_duration / 100
        
        seconds = real / gst.SECOND

        return "%02d:%02d" % (seconds / 60, seconds % 60)

    def scale_button_press_cb(self, widget, event):
        # see seek.c:start_seek
        gst.debug('starting seek')
        
        self.button.set_sensitive(False)
        self.was_playing = self.player.is_playing()
        if self.was_playing:
            self.player.pause()

        # don't timeout-update position during seek
        if self.update_id != -1:
            gobject.source_remove(self.update_id)
            self.update_id = -1

        # make sure we get changed notifies
        if self.changed_id == -1:
            self.changed_id = self.hscale.connect('value-changed',
                self.scale_value_changed_cb)
            
    def scale_value_changed_cb(self, scale):
        # see seek.c:seek_cb
        real = long(scale.get_value() * self.p_duration / 100) # in ns
        gst.debug('value changed, perform seek to %r' % real)
        self.player.seek(real)
        # allow for a preroll
        self.player.get_state(timeout=50*gst.MSECOND) # 50 ms

    def scale_button_release_cb(self, widget, event):
        # see seek.cstop_seek
        widget.disconnect(self.changed_id)
        self.changed_id = -1

        self.button.set_sensitive(True)
        if self.seek_timeout_id != -1:
            gobject.source_remove(self.seek_timeout_id)
            self.seek_timeout_id = -1
        else:
            gst.debug('released slider, setting back to playing')
            if self.was_playing:
                self.player.play()

        if self.update_id != -1:
            self.error('Had a previous update timeout id')
        else:
            self.update_id = gobject.timeout_add(self.UPDATE_INTERVAL,
                self.update_scale_cb)

    def update_scale_cb(self):
        self.p_position, self.p_duration = self.player.query_position()
        if self.p_position != gst.CLOCK_TIME_NONE:
            value = self.p_position * 100.0 / self.p_duration
            self.adjustment.set_value(value)

        return True

def main(args):
    def usage():
        sys.stderr.write("usage: %s URI-OF-MEDIA-FILE\n" % args[0])
        sys.exit(1)

    # Need to register our derived widget types for implicit event
    # handlers to get called.
    gobject.type_register(PlayerWindow)
    gobject.type_register(VideoWidget)

    w = PlayerWindow()

    if len(args) != 2:
        usage()

    if not gst.uri_is_valid(args[1]):
        sys.stderr.write("Error: Invalid URI: %s\n" % args[1])
        sys.exit(1)

    w.load_file(args[1])
    w.show_all()

    gtk.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv))



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2002 David I. Lehn
# Copyright (C) 2004 Johan Dahlin
# Copyright (C) 2005 Edward Hervey
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

from common import gst, unittest, TestCase

import gobject

def find_mixer_element():
    """ Searches for an element implementing the mixer interface """
    allmix = [x for x in gst.registry_get_default().get_feature_list(gst.ElementFactory)
              if x.has_interface("GstMixer") and x.has_interface("GstPropertyProbe")]
    if allmix == []:
        return None
    return allmix[0]

class Availability(TestCase):
    def testXOverlay(self):
        assert hasattr(gst.interfaces, 'XOverlay')
        assert issubclass(gst.interfaces.XOverlay, gobject.GInterface)

    def testMixer(self):
        assert hasattr(gst.interfaces, 'Mixer')
        assert issubclass(gst.interfaces.Mixer, gobject.GInterface)

class FunctionCall(TestCase):
    def FIXME_testXOverlay(self):
        # obviously a testsuite is not allowed to instantiate this
        # since it needs a running X or will fail.  find some way to
        # deal with that.
        element = gst.element_factory_make('xvimagesink')
        assert isinstance(element, gst.Element)
        assert isinstance(element, gst.interfaces.XOverlay)
        element.set_xwindow_id(0L)

class MixerTest(TestCase):
    def setUp(self):
        TestCase.setUp(self)
        amix = find_mixer_element()
        if amix:
            self.mixer = amix.create()
        else:
            self.mixer = None

    def tearDown(self):
        del self.mixer
        TestCase.tearDown(self)

    def testGetProperty(self):
        if self.mixer == None:
            return
        self.failUnless(self.mixer.probe_get_property('device'))
        self.assertRaises(ValueError,
                          self.mixer.probe_get_property, 'non-existent')

    def testGetProperties(self):
        if self.mixer == None:
            return
        properties = self.mixer.probe_get_properties()
        self.failUnless(properties)
        self.assertEqual(type(properties), list)
        prop = properties[0]
        self.assertEqual(prop.name, 'device')
        self.assertEqual(prop.value_type, gobject.TYPE_STRING)

    def testGetValuesName(self):
        if self.mixer == None:
            return
        values = self.mixer.probe_get_values_name('device')
        self.assertEqual(type(values), list)


if __name__ == "__main__":
    unittest.main()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

import sys

import gst
import gst.interfaces

pipeline = "alsasrc"
if sys.argv[1:]:
    pipeline = " ".join(sys.argv[1:])
a = gst.element_factory_make(pipeline)
print dir(a)

res = a.set_state(gst.STATE_PAUSED)
if res != gst.STATE_CHANGE_SUCCESS:
    print "Could not set pipeline %s to PAUSED" % pipeline

print "Inputs:"
for t in a.list_tracks():
    if t.flags & gst.interfaces.MIXER_TRACK_INPUT:
        sys.stdout.write(t.label)
    sys.stdout.write(': %d - %d' % (t.min_volume, t.max_volume))
    volumes = a.get_volume(t)
    sys.stdout.write(': %r' % (volumes, ))
    if t.props.num_channels > 0:
        a.set_volume(t, volumes=volumes)
    if t.flags & gst.interfaces.MIXER_TRACK_RECORD:
        sys.stdout.write(' (selected)')
    sys.stdout.write('\n')




#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# gst-python
# Copyright (C) 2002 David I. Lehn <dlehn@users.sourceforge.net>
#               2004 Johan Dahlin  <johan@gnome.org>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
# 
# Author: David I. Lehn <dlehn@users.sourceforge.net>
#

import sys

import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')
import gst


mainloop = gobject.MainLoop()

def on_eos(bus, msg):
   mainloop.quit()

def filter(input, output):
   "A GStreamer copy pipeline which can add arbitrary filters"

   # create a new bin to hold the elements
   bin = gst.parse_launch('filesrc name=source ! ' +
                          'progressreport ! ' +
                           # This 'statistics' element is depreciated in 0.10
                          #'statistics silent=false buffer-update-freq=1 ' +
                          #'update_on_eos=true ! ' +
                          'filesink name=sink')
   filesrc = bin.get_by_name('source')
   filesrc.set_property('location', input)

   filesink = bin.get_by_name('sink')
   filesink.set_property('location', output)

   bus = bin.get_bus()
   bus.add_signal_watch()
   bus.connect('message::eos', on_eos)

   # start playing
   bin.set_state(gst.STATE_PLAYING)

   try:
      mainloop.run()
   except KeyboardInterrupt:
      pass

   # stop the bin
   bin.set_state(gst.STATE_NULL)

def main(args):
   "A GStreamer based cp(1) with stats"

   if len(args) != 3:
      print 'usage: %s source dest' % (sys.argv[0])
      return -1

   return filter(args[1], args[2])

if __name__ == '__main__':
   sys.exit(main(sys.argv))



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2005 Thomas Vander Stichele
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

from common import gst, unittest, TestCase

import gobject
import time
import sys

class BusSignalTest(TestCase):
    def testGoodConstructor(self):
        loop = gobject.MainLoop()
        gst.info ("creating pipeline")
        pipeline = gst.parse_launch("fakesrc ! fakesink")
        gst.info ("getting bus")
        bus = pipeline.get_bus()
        gst.info ("got bus")
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        self.assertEquals(bus.__gstrefcount__, 2)
        self.assertEquals(pipeline.__gstrefcount__, 1)
        gst.info ("about to add a watch on the bus")
        watch_id = bus.connect("message", self._message_received, pipeline, loop, "one")
        bus.add_signal_watch()
        gst.info ("added a watch on the bus")
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        self.assertEquals(bus.__gstrefcount__, 3)
        self.assertEquals(pipeline.__gstrefcount__, 1)

        gst.info("setting to playing")
        ret = pipeline.set_state(gst.STATE_PLAYING)
        gst.info("set to playing %s, loop.run" % ret)
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        loop.run()

        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        gst.info("setting to paused")
        ret = pipeline.set_state(gst.STATE_PAUSED)
        gst.info("set to paused %s, loop.run" % ret)
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        loop.run()
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))

        gst.info("setting to ready")
        ret = pipeline.set_state(gst.STATE_READY)
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        gst.info("set to READY %s, loop.run" % ret)
        loop.run()

        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        gst.info("setting to NULL")
        ret = pipeline.set_state(gst.STATE_NULL)
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        gst.info("set to NULL %s" % ret)
        self.gccollect()
        self.assertEquals(bus.__gstrefcount__, 3)
        # FIXME: state change thread needs to die
        while pipeline.__gstrefcount__ > 1:
            gst.debug('waiting for pipeline refcount to drop')
            time.sleep(0.1)
        self.assertEquals(pipeline.__gstrefcount__, 1)

        gst.info("about to remove the watch id")
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        bus.remove_signal_watch()
        gst.info("bus watch id removed")
        bus.disconnect(watch_id)
        gst.info("disconnected callback")
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        self.gccollect()
        gst.info("pipeliner:%d/%d busr:%d" % (pipeline.__gstrefcount__, pipeline.__grefcount__, bus.__gstrefcount__))
        
        self.assertEquals(bus.__gstrefcount__, 2)
        self.assertEquals(pipeline.__gstrefcount__, 1)

        gst.info("removing pipeline")
        del pipeline
        gst.info("pipeline removed")
        gst.info("busr:%d" % bus.__gstrefcount__)

        self.gccollect()

        # flush the bus
        bus.set_flushing(True)
        bus.set_flushing(False)
        self.gccollect()
        # FIXME: refcount is still 2
        self.assertEquals(bus.__gstrefcount__, 1)

    def _message_received(self, bus, message, pipeline, loop, id):
        self.failUnless(isinstance(bus, gst.Bus))
        self.failUnless(isinstance(message, gst.Message))
        self.assertEquals(id, "one")
        loop.quit()
        return True

    def testSyncHandlerCallbackRefcount(self):
        def callback1():
            pass

        def callback2():
            pass

        bus = gst.Bus()

        # set
        self.failUnless(sys.getrefcount(callback1), 2)
        bus.set_sync_handler(callback1)
        self.failUnless(sys.getrefcount(callback1), 3)

        # set again
        self.failUnless(sys.getrefcount(callback1), 3)
        bus.set_sync_handler(callback1)
        self.failUnless(sys.getrefcount(callback1), 3)

        # replace
        # this erros out in gst_bus_set_sync_handler, but we need to check that
        # we don't leak anyway
        self.failUnless(sys.getrefcount(callback2), 2)
        bus.set_sync_handler(callback2)
        self.failUnless(sys.getrefcount(callback1), 2)
        self.failUnless(sys.getrefcount(callback2), 3)

        # unset
        bus.set_sync_handler(None)
        self.failUnless(sys.getrefcount(callback2), 2)

class BusAddWatchTest(TestCase):

    def testADumbExample(self):
        gst.info("creating pipeline")
        pipeline = gst.parse_launch("fakesrc ! fakesink")
        gst.info("pipeliner:%s" % pipeline.__gstrefcount__)
        bus = pipeline.get_bus()
        gst.info("got bus, pipeliner:%d, busr:%d" % (pipeline.__gstrefcount__,
                                                     bus.__gstrefcount__))
##         watch_id = bus.add_watch(self._message_received, pipeline)
##         gst.info("added watch,  pipeliner:%d, busr:%d" % (pipeline.__gstrefcount__,
##                                                      bus.__gstrefcount__))
##         gobject.source_remove(watch_id)
##         gst.info("removed watch,  pipeliner:%d, busr:%d" % (pipeline.__gstrefcount__,
##                                                      bus.__gstrefcount__))
        
        
    def testGoodConstructor(self):
        loop = gobject.MainLoop()
        gst.info ("creating pipeline")
        pipeline = gst.parse_launch("fakesrc ! fakesink")
        gst.info ("getting bus")
        bus = pipeline.get_bus()
        gst.info ("got bus")
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        self.assertEquals(bus.__gstrefcount__, 2)
        self.assertEquals(pipeline.__gstrefcount__, 1)
        gst.info ("about to add a watch on the bus")
        watch_id = bus.add_watch(self._message_received, pipeline, loop, "one")
        gst.info ("added a watch on the bus")
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        self.assertEquals(bus.__gstrefcount__, 3)
        self.assertEquals(pipeline.__gstrefcount__, 1)

        gst.info("setting to playing")
        ret = pipeline.set_state(gst.STATE_PLAYING)
        gst.info("set to playing %s, loop.run" % ret)
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        loop.run()

        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        gst.info("setting to paused")
        ret = pipeline.set_state(gst.STATE_PAUSED)
        gst.info("set to paused %s, loop.run" % ret)
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        loop.run()
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))

        gst.info("setting to ready")
        ret = pipeline.set_state(gst.STATE_READY)
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        gst.info("set to READY %s, loop.run" % ret)
        loop.run()

        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        gst.info("setting to NULL")
        ret = pipeline.set_state(gst.STATE_NULL)
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        gst.info("set to NULL %s" % ret)
        self.gccollect()
        self.assertEquals(bus.__gstrefcount__, 3)
        self.assertEquals(pipeline.__gstrefcount__, 1)

        gst.info("about to remove the watch id")
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        self.failUnless(gobject.source_remove(watch_id))
        gst.info("bus watch id removed")
        gst.info("pipeliner:%d busr:%d" % (pipeline.__gstrefcount__, bus.__gstrefcount__))
        self.gccollect()
        gst.info("pipeliner:%d/%d busr:%d" % (pipeline.__gstrefcount__, pipeline.__grefcount__, bus.__gstrefcount__))
        
        self.assertEquals(bus.__gstrefcount__, 2)
        self.assertEquals(pipeline.__gstrefcount__, 1)

        gst.info("removing pipeline")
        del pipeline
        gst.info("pipeline removed")
        gst.info("busr:%d" % bus.__gstrefcount__)

        self.gccollect()

        # flush the bus
        bus.set_flushing(True)
        bus.set_flushing(False)
        self.gccollect()
        # FIXME: refcount is still 2
        self.assertEquals(bus.__gstrefcount__, 1)

    def _message_received(self, bus, message, pipeline, loop, id):
        self.failUnless(isinstance(bus, gst.Bus))
        self.failUnless(isinstance(message, gst.Message))
        self.assertEquals(id, "one")
        # doesn't the following line stop the mainloop before the end of the state change ?
        loop.quit()
        return True

        
if __name__ == "__main__":
    unittest.main()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2002 David I. Lehn
# Copyright (C) 2004 Johan Dahlin
# Copyright (C) 2005 Edward Hervey
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import sys
import gc
from common import gobject, gst, unittest, TestCase

class BufferTest(TestCase):
    def testBufferBuffer(self):
        buf = gst.Buffer('test')
        assert str(buffer(buf)) == 'test'

    def testBufferStr(self):
        buffer = gst.Buffer('test')
        assert str(buffer) == 'test'

    def testBufferAlloc(self):
        bla = 'mooooooo'
        buffer = gst.Buffer(bla + '12345')
        gc.collect ()
        assert str(buffer) == 'mooooooo12345'
		
    def testBufferBadConstructor(self):
        self.assertRaises(TypeError, gst.Buffer, 'test', 0)
        
    def testBufferStrNull(self):
        test_string = 't\0e\0s\0t\0'
        buffer = gst.Buffer(test_string)
        assert str(buffer) == test_string

    def testBufferSize(self):
        test_string = 'a little string'
        buffer = gst.Buffer(test_string)
        assert len(buffer) == len(test_string)
        assert hasattr(buffer, 'size')
        assert buffer.size == len(buffer)
        
    def testBufferCreateSub(self):
        s = ''
        for i in range(64):
            s += '%02d' % i
            
        buffer = gst.Buffer(s)
        self.assertEquals(len(buffer), 128)

        sub = buffer.create_sub(16, 16)
        self.assertEquals(sub.size, 16)
        self.assertEquals(sub.data, buffer.data[16:32])
        self.assertEquals(sub.offset, gst.CLOCK_TIME_NONE)

    def testBufferMerge(self):
        buffer1 = gst.Buffer('foo')
        buffer2 = gst.Buffer('bar')

        merged_buffer = buffer1.merge(buffer2)
        assert str(merged_buffer) == 'foobar'
        
    def testBufferJoin(self):
        buffer1 = gst.Buffer('foo')
        buffer2 = gst.Buffer('bar')

        joined_buffer = buffer1.merge(buffer2)
        assert str(joined_buffer) == 'foobar'
        
    def testBufferSpan(self):
        buffer1 = gst.Buffer('foo')
        buffer2 = gst.Buffer('bar')

        spaned_buffer = buffer1.span(0L, buffer2, 6L)
        assert str(spaned_buffer) == 'foobar'
    def testBufferCopyOnWrite(self):
        s='test_vector'
        buffer = gst.Buffer(s)
        sub = buffer.create_sub(0, buffer.size)
        self.assertEquals(sub.size, buffer.size)
        out = sub.copy_on_write ()
        self.assertEquals(out.size, sub.size)
        assert str(out) == str(buffer)
        out[5] = 'w'
        assert str(out) == 'test_wector'

    def testBufferFlagIsSet(self):
        buffer = gst.Buffer()
        # Off by default
        assert not buffer.flag_is_set(gst.BUFFER_FLAG_READONLY)

        # Try switching on and off
        buffer.flag_set(gst.BUFFER_FLAG_READONLY)
        assert buffer.flag_is_set(gst.BUFFER_FLAG_READONLY)
        buffer.flag_unset(gst.BUFFER_FLAG_READONLY)
        assert not buffer.flag_is_set(gst.BUFFER_FLAG_READONLY)

        # Try switching on and off
        buffer.flag_set(gst.BUFFER_FLAG_IN_CAPS)
        assert buffer.flag_is_set(gst.BUFFER_FLAG_IN_CAPS)
        buffer.flag_unset(gst.BUFFER_FLAG_IN_CAPS)
        assert not buffer.flag_is_set(gst.BUFFER_FLAG_IN_CAPS)

    def testAttrFlags(self):
        buffer = gst.Buffer()
        assert hasattr(buffer, "flags")
        assert isinstance(buffer.flags, int)
 
    def testAttrTimestamp(self):
        buffer = gst.Buffer()
        assert hasattr(buffer, "timestamp")
        assert isinstance(buffer.timestamp, long)

        assert buffer.timestamp == gst.CLOCK_TIME_NONE
        buffer.timestamp = 0
        assert buffer.timestamp == 0
        buffer.timestamp = 2**64 - 1
        assert buffer.timestamp == 2**64 - 1

    def testAttrDuration(self):
        buffer = gst.Buffer()
        assert hasattr(buffer, "duration")
        assert isinstance(buffer.duration, long)

        assert buffer.duration == gst.CLOCK_TIME_NONE
        buffer.duration = 0
        assert buffer.duration == 0
        buffer.duration = 2**64 - 1
        assert buffer.duration == 2**64 - 1
        
    def testAttrOffset(self):
        buffer = gst.Buffer()
        assert hasattr(buffer, "offset")
        assert isinstance(buffer.offset, long)

        assert buffer.offset == gst.CLOCK_TIME_NONE
        buffer.offset = 0
        assert buffer.offset == 0
        buffer.offset = 2**64 - 1
        assert buffer.offset == 2**64 - 1

    def testAttrOffset_end(self):
        buffer = gst.Buffer()
        assert hasattr(buffer, "offset_end")
        assert isinstance(buffer.offset_end, long)

        assert buffer.offset_end == gst.CLOCK_TIME_NONE
        buffer.offset_end = 0
        assert buffer.offset_end == 0
        buffer.offset_end = 2**64 - 1
        assert buffer.offset_end == 2**64 - 1

    def testBufferCaps(self):
        buffer = gst.Buffer()
        caps = gst.caps_from_string('foo/blah')
        gst.info("before settings caps")
        buffer.set_caps(caps)
        gst.info("after settings caps")
        c = buffer.get_caps()
        gst.info("after getting caps")
        self.assertEquals(caps, c)

if __name__ == "__main__":
    unittest.main()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2002 David I. Lehn
# Copyright (C) 2004 Johan Dahlin
# Copyright (C) 2005 Edward Hervey
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import time

from common import gst, unittest, TestCase, pygobject_2_13

import gobject

class TestConstruction(TestCase):
    def setUp(self):
        self.gctrack()

    def tearDown(self):
        self.gccollect()
        self.gcverify()

    def testGoodConstructor(self):
        name = 'test-pipeline'
        pipeline = gst.Pipeline(name)
        self.assertEquals(pipeline.__gstrefcount__, 1)
        assert pipeline is not None, 'pipeline is None'
        self.failUnless(isinstance(pipeline, gst.Pipeline),
            'pipeline is not a GstPipline')
        assert pipeline.get_name() == name, 'pipelines name is wrong'
        self.assertEquals(pipeline.__gstrefcount__, 1)

    def testParseLaunch(self):
        pipeline = gst.parse_launch('fakesrc ! fakesink')
        
class Pipeline(TestCase):
    def setUp(self):
        self.gctrack()
        self.pipeline = gst.Pipeline('test-pipeline')
        source = gst.element_factory_make('fakesrc', 'source')
        source.set_property('num-buffers', 5)
        sink = gst.element_factory_make('fakesink', 'sink')
        self.pipeline.add(source, sink)
        gst.element_link_many(source, sink)

    def tearDown(self):
        del self.pipeline
        self.gccollect()
        self.gcverify()

    def testRun(self):
        self.assertEqual(self.pipeline.get_state()[1], gst.STATE_NULL)
        self.pipeline.set_state(gst.STATE_PLAYING)
        self.assertEqual(self.pipeline.get_state()[1], gst.STATE_PLAYING)
        
        time.sleep(1)

        self.assertEqual(self.pipeline.get_state()[1], gst.STATE_PLAYING)
        self.pipeline.set_state(gst.STATE_NULL)
        self.assertEqual(self.pipeline.get_state()[1], gst.STATE_NULL)

class PipelineTags(TestCase):
    def setUp(self):
        self.gctrack()
        self.pipeline = gst.parse_launch('audiotestsrc num-buffers=100 ! vorbisenc name=encoder ! oggmux name=muxer ! fakesink')

    def tearDown(self):
        del self.pipeline
        self.gccollect()
        self.gcverify()

    def testRun(self):
        # in 0.10.15.1, this triggers
        # sys:1: gobject.Warning: g_value_get_uint: assertion `G_VALUE_HOLDS_UINT (value)' failed
        # during pipeline playing

        l = gst.TagList()
        l[gst.TAG_ARTIST] = 'artist'
        l[gst.TAG_TRACK_NUMBER] = 1
        encoder = self.pipeline.get_by_name('encoder')
        encoder.merge_tags(l, gst.TAG_MERGE_APPEND)

        self.assertEqual(self.pipeline.get_state()[1], gst.STATE_NULL)
        self.pipeline.set_state(gst.STATE_PLAYING)
        self.assertEqual(self.pipeline.get_state()[1], gst.STATE_PLAYING)
        
        time.sleep(1)

        self.assertEqual(self.pipeline.get_state()[1], gst.STATE_PLAYING)
        self.pipeline.set_state(gst.STATE_NULL)
        self.assertEqual(self.pipeline.get_state()[1], gst.STATE_NULL)


class Bus(TestCase):
    def testGet(self):
        pipeline = gst.Pipeline('test')
        self.assertEquals(pipeline.__gstrefcount__, 1)
        bus = pipeline.get_bus()
        self.assertEquals(pipeline.__gstrefcount__, 1)
        # one for python and one for the pipeline
        self.assertEquals(bus.__gstrefcount__, 2)

        del pipeline
		if not pygobject_2_13:
            self.failUnless(self.gccollect())
        self.assertEquals(bus.__gstrefcount__, 1)
        
class PipelineAndBus(TestCase):
    def setUp(self):
        TestCase.setUp(self)
        self.pipeline = gst.Pipeline('test-pipeline')
        source = gst.element_factory_make('fakesrc', 'source')
        sink = gst.element_factory_make('fakesink', 'sink')
        self.pipeline.add(source, sink)
        gst.element_link_many(source, sink)

        self.bus = self.pipeline.get_bus()
        self.assertEquals(self.bus.__gstrefcount__, 2)
        self.handler = self.bus.add_watch(self._message_received)
        self.assertEquals(self.bus.__gstrefcount__, 3)
        self.assertEquals(self.pipeline.__gstrefcount__, 1)

        self.loop = gobject.MainLoop()

    def tearDown(self):
        # FIXME: fix the refcount issues with the bus/pipeline
        # flush the bus to be able to assert on the pipeline refcount
        #while self.pipeline.__gstrefcount__ > 1:
        self.gccollect()

        # one for the pipeline, two for the snake
        # three for the watch now shake shake shake but don't you
        self.assertEquals(self.bus.__gstrefcount__, 3)
        self.failUnless(gobject.source_remove(self.handler))
        self.assertEquals(self.bus.__gstrefcount__, 2)
        self.gccollect()

        gst.debug('THOMAS: pipeline rc %d' % self.pipeline.__gstrefcount__)
        #self.assertEquals(self.pipeline.__gstrefcount__, 1)
        del self.pipeline
        self.gccollect()
        #self.assertEquals(self.bus.__gstrefcount__, 2)
        del self.bus
        self.gccollect()

        # the async thread can be holding a ref, Wim is going to work on this
        #TestCase.tearDown(self)

    def _message_received(self, bus, message):
        gst.debug('received message: %s, %s' % (
            message.src.get_path_string(), message.type.value_nicks[1]))
        t = message.type
        if t == gst.MESSAGE_STATE_CHANGED:
            old, new, pen = message.parse_state_changed()
            gst.debug('%r state change from %r to %r' % (
                message.src.get_path_string(), old, new))
            if message.src == self.pipeline and new == self.final:
                self.loop.quit()

        return True

    def testPlaying(self):
        self.final = gst.STATE_PLAYING
        ret = self.pipeline.set_state(gst.STATE_PLAYING)
        self.assertEquals(ret, gst.STATE_CHANGE_ASYNC)

        # go into a main loop to wait for messages
        self.loop.run()

        # we go to READY so we get messages; going to NULL would set
        # the bus flushing
        self.final = gst.STATE_READY
        ret = self.pipeline.set_state(gst.STATE_READY)
        self.assertEquals(ret, gst.STATE_CHANGE_SUCCESS)
        self.loop.run()

        # FIXME: not setting to NULL causes a deadlock; we might want to
        # fix this in the bindings
        self.assertEquals(self.pipeline.set_state(gst.STATE_NULL),
            gst.STATE_CHANGE_SUCCESS)
        self.assertEquals(self.pipeline.get_state(),
            (gst.STATE_CHANGE_SUCCESS, gst.STATE_NULL, gst.STATE_VOID_PENDING))
        self.gccollect()

class TestPipeSub(gst.Pipeline):
    def do_handle_message(self, message):
        self.debug('do_handle_message')
        gst.Pipeline.do_handle_message(self, message)
        self.type = message.type
gobject.type_register(TestPipeSub)

class TestPipeSubSub(TestPipeSub):
    def do_handle_message(self, message):
        self.debug('do_handle_message')
        TestPipeSub.do_handle_message(self, message)
gobject.type_register(TestPipeSubSub)


# see http://bugzilla.gnome.org/show_bug.cgi?id=577735
class TestSubClass(TestCase):
    def setUp(self):
        self.gctrack()

    def tearDown(self):
        self.gccollect()
        self.gcverify()

    def testSubClass(self):
        p = TestPipeSub()
        u = gst.element_factory_make('uridecodebin')
        self.assertEquals(u.__grefcount__, 1)
        self.failIf(getattr(p, 'type', None))
        # adding uridecodebin triggers a clock-provide message;
        # this message should be dropped, and thus not affect
        # the refcount of u beyond the parenting.
        p.add(u)
        self.assertEquals(getattr(p, 'type', None), gst.MESSAGE_CLOCK_PROVIDE)
        self.assertEquals(u.__grefcount__, 2)
        del p
        self.assertEquals(u.__grefcount__, 1)

    def testSubSubClass(self):
        # Edward is worried that a subclass of a subclass will screw up
        # the refcounting wrt. GST_BUS_DROP
        p = TestPipeSubSub()
        u = gst.element_factory_make('uridecodebin')
        self.assertEquals(u.__grefcount__, 1)
        self.failIf(getattr(p, 'type', None))
        p.add(u)
        self.assertEquals(getattr(p, 'type', None), gst.MESSAGE_CLOCK_PROVIDE)
        self.assertEquals(u.__grefcount__, 2)
        del p
        self.assertEquals(u.__grefcount__, 1)
     
if __name__ == "__main__":
    unittest.main()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2024 Collabora Ltd
#  Author: Olivier Crête <olivier.crete@collabora.com>
# Copyright (C) 2024 Intel Corporation
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import overrides_hack
overrides_hack

import gi
gi.require_version("GLib", "2.0")
gi.require_version("Gst", "1.0")
gi.require_version("GstAnalytics", "1.0")
gi.require_version("GstVideo", "1.0")

from gi.repository import GstVideo
from gi.repository import GstAnalytics
from gi.repository import Gst
from gi.repository import GLib
from common import TestCase

Gst.init([])


class TestAnalyticsODMtd(TestCase):
    def test(self):
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        meta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        self.assertIsNotNone(meta)

        m2 = GstAnalytics.buffer_get_analytics_relation_meta(buf)
        self.assertEqual(meta, m2)

        qk = GLib.quark_from_string("testQuark")

        (ret, mtd) = meta.add_od_mtd(qk, 10, 20, 30, 40, 0.3)
        self.assertTrue(ret)
        self.assertIsNotNone(mtd)

        (ret, mtd) = meta.get_od_mtd(0)
        self.assertTrue(ret)
        self.assertIsNotNone(mtd)

        # Ensure there is no mtd 1, only 0
        (ret, _) = meta.get_mtd(1, GstAnalytics.MTD_TYPE_ANY)
        self.assertFalse(ret)

        # The is only one od mtd
        (ret, _) = meta.get_od_mtd(1)
        self.assertFalse(ret)

        # There is no Class mtd
        (ret, _) = meta.get_cls_mtd(0)
        self.assertFalse(ret)

        # meta and m2 should return the same tuple
        self.assertEqual(meta.get_od_mtd(0)[1].get_location(),
                         m2.get_od_mtd(0)[1].get_location())

        self.assertEqual(mtd.get_obj_type(), qk)

        location = meta.get_od_mtd(0)[1].get_location()
        self.assertEqual(location[1], 10)
        self.assertEqual(location[2], 20)
        self.assertEqual(location[3], 30)
        self.assertEqual(location[4], 40)
        self.assertAlmostEqual(location[5], 0.3, 3)

        location = meta.get_od_mtd(0)[1].get_oriented_location()
        self.assertEqual(location[1], 10)
        self.assertEqual(location[2], 20)
        self.assertEqual(location[3], 30)
        self.assertEqual(location[4], 40)
        self.assertEqual(location[5], 0)
        self.assertAlmostEqual(location[6], 0.3, 3)

        (ret, mtd) = meta.add_oriented_od_mtd(qk, 600, 400, 200, 100, 0.785, 0.3)
        self.assertTrue(ret)
        self.assertIsNotNone(mtd)

        (ret, mtd) = meta.get_od_mtd(1)
        self.assertTrue(ret)
        self.assertIsNotNone(mtd)

        location = mtd.get_oriented_location()
        self.assertEqual(location[1], 600)
        self.assertEqual(location[2], 400)
        self.assertEqual(location[3], 200)
        self.assertEqual(location[4], 100)
        self.assertAlmostEqual(location[5], 0.785, 3)
        self.assertAlmostEqual(location[6], 0.3, 3)

        location = mtd.get_location()
        self.assertEqual(location[1], 594)
        self.assertEqual(location[2], 344)
        self.assertEqual(location[3], 212)
        self.assertEqual(location[4], 212)
        self.assertAlmostEqual(location[5], 0.3, 3)


class TestAnalyticsClsMtd(TestCase):
    def test(self):
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        meta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        self.assertIsNotNone(meta)

        qks = (GLib.quark_from_string("q1"),
               GLib.quark_from_string("q2"),
               GLib.quark_from_string("q3"))

        (ret, mtd) = meta.add_cls_mtd([0.1, 0.2, 0.3], qks)
        self.assertTrue(ret)
        self.assertIsNotNone(mtd)

        cnt = mtd.get_length()
        self.assertEqual(cnt, 3)

        for i in range(cnt):
            self.assertEqual(mtd.get_index_by_quark(qks[i]), i)
            self.assertAlmostEqual(mtd.get_level(i), (i + 1) / 10, 7)
            self.assertEqual(mtd.get_quark(i), qks[i])


class TestAnalyticsTrackingMtd(TestCase):
    def test(self):
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        meta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        self.assertIsNotNone(meta)

        (ret, mtd) = meta.add_tracking_mtd(1, 10)
        self.assertTrue(ret)
        rets = mtd.get_info()
        self.assertFalse(rets.tracking_lost)
        self.assertEqual(rets.tracking_first_seen, 10)
        self.assertEqual(rets.tracking_last_seen, 10)

        mtd.update_last_seen(20)

        rets = mtd.get_info()
        self.assertEqual(rets.tracking_first_seen, 10)
        self.assertEqual(rets.tracking_last_seen, 20)

        mtd.set_lost()
        rets = mtd.get_info()
        self.assertTrue(rets.tracking_lost)


class TestAnalyticsSegmentationMtd(TestCase):
    def test(self):
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        meta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        self.assertIsNotNone(meta)

        mask_buf = Gst.Buffer.new_allocate(None, 100, None)
        GstVideo.buffer_add_video_meta(mask_buf,
                                       GstVideo.VideoFrameFlags.NONE,
                                       GstVideo.VideoFormat.GRAY8, 10, 10)

        (ret, mtd) = meta.add_segmentation_mtd(mask_buf,
                                               GstAnalytics.SegmentationType.SEMANTIC,
                                               [7, 4, 2], 0, 0, 7, 13)
        self.assertTrue(ret)

        self.assertEqual((mask_buf, 0, 0, 7, 13), mtd.get_mask())
        self.assertEqual(mtd.get_region_count(), 3)
        self.assertEqual(mtd.get_region_id(0), 7)
        self.assertEqual(mtd.get_region_id(1), 4)
        self.assertEqual(mtd.get_region_id(2), 2)

        self.assertEqual(mtd.get_region_index(1), (False, 0))
        self.assertEqual(mtd.get_region_index(7), (True, 0))
        self.assertEqual(mtd.get_region_index(4), (True, 1))
        self.assertEqual(mtd.get_region_index(2), (True, 2))


class TestAnalyticsTensorMeta(TestCase):
    def test(self):
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        tmeta = GstAnalytics.buffer_add_tensor_meta(buf)
        self.assertIsNotNone(tmeta)

        data = Gst.Buffer.new_allocate(None, 2 * 3 * 4)
        self.assertIsNotNone(data)

        tensor = GstAnalytics.Tensor.new_simple(0, GstAnalytics.TensorDataType.UINT8,
                                                data,
                                                GstAnalytics.TensorDimOrder.ROW_MAJOR,
                                                [1, 2, 3, 4])
        self.assertIsNotNone(tensor)
        self.assertEqual(tensor.id, 0)
        self.assertEqual(tensor.num_dims, 4)
        dims = tensor.get_dims()
        self.assertEqual(len(dims), 4)
        self.assertEqual(dims[0], 1)
        self.assertEqual(dims[1], 2)
        self.assertEqual(dims[2], 3)
        self.assertEqual(dims[3], 4)
        self.assertEqual(tensor.data, data)
        self.assertEqual(tensor.data_type, GstAnalytics.TensorDataType.UINT8)
        self.assertEqual(tensor.dims_order, GstAnalytics.TensorDimOrder.ROW_MAJOR)

        data2 = Gst.Buffer.new_allocate(None, 2 * 3 * 4 * 5)
        tensor2 = GstAnalytics.Tensor.new_simple(0, GstAnalytics.TensorDataType.UINT16,
                                                 data2,
                                                 GstAnalytics.TensorDimOrder.ROW_MAJOR,
                                                 [1, 3, 4, 5])
        tmeta.set([tensor, tensor2])

        tmeta2 = GstAnalytics.buffer_get_tensor_meta(buf)
        self.assertEqual(tmeta2.num_tensors, 2)
        self.assertEqual(tmeta2.get(0).data, data)
        self.assertEqual(tmeta2.get(1).data, data2)

        data3 = Gst.Buffer.new_allocate(None, 30)
        tensor3 = GstAnalytics.Tensor.new_simple(0,
                                                 GstAnalytics.TensorDataType.UINT16,
                                                 data3,
                                                 GstAnalytics.TensorDimOrder.ROW_MAJOR,
                                                 [0, 2, 5])
        self.assertIsNotNone(tensor3)


class TestAnalyticsRelationMetaIterator(TestCase):
    def test(self):
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        rmeta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        self.assertIsNotNone(rmeta)

        mask_buf = Gst.Buffer.new_allocate(None, 100, None)
        GstVideo.buffer_add_video_meta(mask_buf,
                                       GstVideo.VideoFrameFlags.NONE,
                                       GstVideo.VideoFormat.GRAY8, 10, 10)

        (_, od_mtd) = rmeta.add_od_mtd(GLib.quark_from_string("od"), 1, 1, 2, 2, 0.1)
        (_, od_mtd1) = rmeta.add_od_mtd(GLib.quark_from_string("od"), 1, 1, 2, 2, 0.1)
        (_, od_mtd2) = rmeta.add_od_mtd(GLib.quark_from_string("od"), 1, 1, 2, 2, 0.1)
        (_, cls_mtd) = rmeta.add_one_cls_mtd(0.1, GLib.quark_from_string("cls"))
        (_, cls_mtd1) = rmeta.add_one_cls_mtd(0.4, GLib.quark_from_string("cls"))
        (_, trk_mtd) = rmeta.add_tracking_mtd(1, 10)
        (_, trk_mtd1) = rmeta.add_tracking_mtd(1, 11)
        (_, seg_mtd) = rmeta.add_segmentation_mtd(mask_buf,
                                                  GstAnalytics.SegmentationType.SEMANTIC,
                                                  [7, 4, 2], 0, 0, 7, 13)

        mtds = [
            (od_mtd, GstAnalytics.ODMtd.get_mtd_type()),
            (od_mtd1, GstAnalytics.ODMtd.get_mtd_type()),
            (od_mtd2, GstAnalytics.ODMtd.get_mtd_type()),
            (cls_mtd, GstAnalytics.ClsMtd.get_mtd_type()),
            (cls_mtd1, GstAnalytics.ClsMtd.get_mtd_type()),
            (trk_mtd, GstAnalytics.TrackingMtd.get_mtd_type()),
            (trk_mtd1, GstAnalytics.TrackingMtd.get_mtd_type()),
            (seg_mtd, GstAnalytics.SegmentationMtd.get_mtd_type())
        ]

        od_index_mtds = [0, 1, 2]
        cls_index_mtds = [3, 4]
        trk_index_mtds = [5, 6]
        seg_index_mtds = [7]

        mtds_from_iter = list(rmeta)

        self.assertEqual(len(mtds), len(mtds_from_iter))

        # Iterating on type GstAnalytics.ODMtd
        for j, i in zip(od_index_mtds, rmeta.iter_on_type(GstAnalytics.ODMtd)):
            assert mtds[j][0] == i
            assert mtds[j][0].id == i.id
            assert mtds[j][0].meta == i.meta
            assert mtds[j][1] == i.get_mtd_type()
            # call a method to ensure it's a ODMtd
            loc = i.get_location()

        # Iterating on type GstAnalytics.ClsMtd
        for j, i in zip(cls_index_mtds, rmeta.iter_on_type(GstAnalytics.ClsMtd)):
            assert mtds[j][0] == i
            assert mtds[j][0].id == i.id
            assert mtds[j][0].meta == i.meta
            assert mtds[j][1] == i.get_mtd_type()
            # call a method to ensure it's a ClsMtd
            level = i.get_level(0)

        # Iterating on type GstAnalytics.TrackingMtd
        for j, i in zip(trk_index_mtds, rmeta.iter_on_type(GstAnalytics.TrackingMtd)):
            assert mtds[j][0] == i
            assert mtds[j][0].id == i.id
            assert mtds[j][0].meta == i.meta
            assert mtds[j][1] == i.get_mtd_type()
            # call a method to ensure it's a TrackingMtd
            info = i.get_info()

        # Iterating on type GstAnalytics.SegmentationMtd
        for j, i in zip(seg_index_mtds, rmeta.iter_on_type(GstAnalytics.SegmentationMtd)):
            assert mtds[j][0] == i
            assert mtds[j][0].id == i.id
            assert mtds[j][0].meta == i.meta
            assert mtds[j][1] == i.get_mtd_type()
            # call a method to ensure it's a SegmentationMtd
            mask = i.get_mask()

        # Iterating on all type
        for e, i in zip(mtds, rmeta):
            assert i == e[0]
            assert e[0].id == i.id
            assert e[0].meta == i.meta
            assert e[1] == i.get_mtd_type()

        # Validate that the object is really a ODMtd
        location = mtds_from_iter[0].get_location()
        self.assertEqual(location[1], 1)
        self.assertEqual(location[2], 1)
        self.assertEqual(location[3], 2)
        self.assertEqual(location[4], 2)
        self.assertAlmostEqual(location[5], 0.1, 3)

        # Test iteration over direct relation
        rmeta.set_relation(GstAnalytics.RelTypes.RELATE_TO, od_mtd.id, od_mtd1.id)
        rmeta.set_relation(GstAnalytics.RelTypes.IS_PART_OF, od_mtd.id, trk_mtd.id)
        rmeta.set_relation(GstAnalytics.RelTypes.RELATE_TO, od_mtd.id, od_mtd2.id)
        rmeta.set_relation(GstAnalytics.RelTypes.RELATE_TO, od_mtd.id, cls_mtd.id)
        expected_mtd_ids = [od_mtd1.id, od_mtd2.id, cls_mtd.id]
        expected_mtd_type = [GstAnalytics.ODMtd, GstAnalytics.ODMtd, GstAnalytics.ClsMtd]
        count = 0
        # Iterate over all type
        for mtd in od_mtd.iter_direct_related(GstAnalytics.RelTypes.RELATE_TO):
            assert mtd.id == expected_mtd_ids[count]
            assert type(mtd) is expected_mtd_type[count]
            if (type(mtd) is GstAnalytics.ODMtd):
                assert mtd.get_obj_type() == GLib.quark_from_string("od")
            elif (type(mtd) is GstAnalytics.ClsMtd):
                assert mtd.get_quark(0) == GLib.quark_from_string("cls")
            count = count + 1

        assert (count == len(expected_mtd_ids))

        # Iterate over only with type GstAnalytics.ODMtd
        count = 0
        for mtd in od_mtd.iter_direct_related(GstAnalytics.RelTypes.RELATE_TO, GstAnalytics.ODMtd):
            assert mtd.id == expected_mtd_ids[count]
            assert type(mtd) is GstAnalytics.ODMtd
            count = count + 1

        assert (count == 2)

        # Create a relation path as od_mtd -> cls_mtd -> trk_mtd -> seg_mtd
        rmeta.set_relation(GstAnalytics.RelTypes.NONE, od_mtd.id, trk_mtd.id)  # clear relation
        rmeta.set_relation(GstAnalytics.RelTypes.RELATE_TO, cls_mtd.id, trk_mtd.id)
        rmeta.set_relation(GstAnalytics.RelTypes.RELATE_TO, trk_mtd.id, seg_mtd.id)
        count = 0
        expected_rel_ids = [od_mtd.id, cls_mtd.id, trk_mtd.id, seg_mtd.id]
        for i in od_mtd.relation_path(seg_mtd, max_span=4):
            assert i == expected_rel_ids[count]
            count += 1
        assert (count == 4)


class TestModelInfo(TestCase):
    def test_modelinfo_load_not_found(self):
        """Test loading a modelinfo file that doesn't exist"""
        modelinfo = GstAnalytics.ModelInfo.load("/nonexistent/model.onnx")
        # Should return None if file not found
        self.assertIsNone(modelinfo)

    def test_modelinfo_with_temporary_file(self):
        """Test modelinfo API with a temporary modelinfo file"""
        import tempfile
        import os

        # Create a temporary modelinfo file
        modelinfo_content = """
[modelinfo]
version=1.0
group-id=test-model-v1

[input_tensor]
dims=1,224,224,3
dir=input
type=uint8
ranges=0.0,255.0

[output_tensor]
dims=1,1000
dir=output
type=float32
id=output_logits
"""

        # Create temporary file
        with tempfile.NamedTemporaryFile(mode='w', suffix='.modelinfo',
                                         delete=False) as f:
            f.write(modelinfo_content)
            temp_modelinfo = f.name

        try:
            # Remove .modelinfo extension to get model filename
            model_filename = temp_modelinfo[:-10]  # Remove '.modelinfo'

            # Load the modelinfo using ModelInfo.load()
            modelinfo = GstAnalytics.ModelInfo.load(model_filename)
            self.assertIsNotNone(modelinfo)

            # Verify it's a ModelInfo object
            self.assertIsInstance(modelinfo, GstAnalytics.ModelInfo)

            # Test get_version
            version = modelinfo.get_version()
            self.assertEqual(version, "1.0")

            # Test get_group_id
            group_id = modelinfo.get_group_id()
            self.assertEqual(group_id, "test-model-v1")

            # Test get_group_id as quark
            group_id_quark = modelinfo.get_quark_group_id()
            self.assertEqual(group_id_quark, GLib.quark_from_string("test-model-v1"))

            # Test find_tensor_name by name
            tensor_name = modelinfo.find_tensor_name(
                GstAnalytics.ModelInfoTensorDirection.INPUT,
                0,  # index
                "input_tensor",  # in_tensor_name hint
                GstAnalytics.TensorDataType.UINT8,
                [1, 224, 224, 3]  # dims
            )
            self.assertEqual(tensor_name, "input_tensor")

            # Test get_id
            output_id = modelinfo.get_id("output_tensor")
            self.assertEqual(output_id, "output_logits")

            # Test get_id as quark
            output_id_quark = modelinfo.get_quark_id("output_tensor")
            self.assertEqual(output_id_quark, GLib.quark_from_string("output_logits"))

            # Test get_input_scales_offsets
            # Case 1: uint8 input [0, 255] to target range [0, 255] (passthrough)
            # GObject Introspection returns (success, scales, offsets)
            input_mins = [0.0]  # uint8 minimum
            input_maxs = [255.0]  # uint8 maximum
            result = modelinfo.get_input_scales_offsets("input_tensor",
                                                        input_mins, input_maxs)
            self.assertTrue(result[0])  # success
            scales = result[1]
            offsets = result[2]
            self.assertEqual(len(scales), 1)  # scales should have 1 element
            self.assertEqual(len(offsets), 1)  # offsets should have 1 element
            self.assertAlmostEqual(scales[0], 1.0, 6)  # (255-0)/(255-0) = 1.0
            self.assertAlmostEqual(offsets[0], 0.0, 6)  # 0 - 0*1.0 = 0.0

            # Test get_dims_order (should default to row-major)
            dims_order = modelinfo.get_dims_order("input_tensor")
            self.assertEqual(dims_order, GstAnalytics.TensorDimOrder.ROW_MAJOR)

            # Test get_target_ranges (returns arrays of min/max from ranges)
            result = modelinfo.get_target_ranges("input_tensor")
            # ranges field contains "0.0,255.0" so this should succeed
            self.assertTrue(result[0])  # success
            mins = result[1]
            maxs = result[2]
            self.assertEqual(len(mins), 1)  # should have 1 range
            self.assertEqual(len(maxs), 1)  # should have 1 range
            self.assertAlmostEqual(mins[0], 0.0, 6)
            self.assertAlmostEqual(maxs[0], 255.0, 6)

            # Free the modelinfo
            modelinfo.free()

        finally:
            # Clean up temporary file
            if os.path.exists(temp_modelinfo):
                os.unlink(temp_modelinfo)
            if os.path.exists(model_filename):
                os.unlink(model_filename)

    def test_modelinfo_version_major_minor(self):
        """Test modelinfo version string parsing for major and minor versions"""
        import tempfile
        import os

        # Test case: Version 1.0 (current format version)
        modelinfo_content_1_0 = """
[modelinfo]
version=1.0
group-id=test-model-v1

[input_tensor]
dims=1,224,224,3
dir=input
type=uint8

[output_tensor]
dims=1,1000
dir=output
type=float32
id=output_logits
"""

        with tempfile.NamedTemporaryFile(mode='w', suffix='.modelinfo',
                                         delete=False) as f:
            f.write(modelinfo_content_1_0)
            temp_modelinfo = f.name

        try:
            model_filename = temp_modelinfo[:-10]  # Remove '.modelinfo'
            modelinfo = GstAnalytics.ModelInfo.load(model_filename)
            self.assertIsNotNone(modelinfo)

            # Verify version string
            version = modelinfo.get_version()
            self.assertEqual(version, "1.0")

            # Parse version string to verify major and minor components
            version_parts = version.split('.')
            self.assertEqual(len(version_parts), 2)
            major_version = int(version_parts[0])
            minor_version = int(version_parts[1])
            self.assertEqual(major_version, 1)
            self.assertEqual(minor_version, 0)

            modelinfo.free()
        finally:
            if os.path.exists(temp_modelinfo):
                os.unlink(temp_modelinfo)
            if os.path.exists(model_filename):
                os.unlink(model_filename)

    def test_modelinfo_version_major_upgrade_rejected(self):
        """Test that modelinfo with unsupported major version is rejected"""
        import tempfile
        import os

        # Test case: Version 2.0 (unsupported major version)
        # The version check should reject this
        modelinfo_content_2_0 = """
[modelinfo]
version=2.0
group-id=test-model-v2

[input_tensor]
dims=1,224,224,3
dir=input
type=uint8

[output_tensor]
dims=1,1000
dir=output
type=float32
id=output_logits
"""

        with tempfile.NamedTemporaryFile(mode='w', suffix='.modelinfo',
                                         delete=False) as f:
            f.write(modelinfo_content_2_0)
            temp_modelinfo = f.name

        try:
            model_filename = temp_modelinfo[:-10]  # Remove '.modelinfo'
            # Load should fail because version 2.0 is not supported
            modelinfo = GstAnalytics.ModelInfo.load(model_filename)
            self.assertIsNone(modelinfo)
        finally:
            if os.path.exists(temp_modelinfo):
                os.unlink(temp_modelinfo)
            if os.path.exists(model_filename):
                os.unlink(model_filename)

    def test_modelinfo_input_ranges_transformations(self):
        """Test modelinfo get_input_scales_offsets with different input ranges"""
        import tempfile
        import os

        # Create a modelinfo with a tensor that expects normalized [0, 1] range
        modelinfo_content = """
[modelinfo]
version=1.0
group-id=test-model-normalization

[input_normalized]
dims=1,224,224,3
dir=input
type=uint8
ranges=0.0,1.0;0.0,1.0;0.0,1.0
"""

        with tempfile.NamedTemporaryFile(mode='w', suffix='.modelinfo',
                                         delete=False) as f:
            f.write(modelinfo_content)
            temp_modelinfo = f.name

        try:
            model_filename = temp_modelinfo[:-10]
            modelinfo = GstAnalytics.ModelInfo.load(model_filename)
            self.assertIsNotNone(modelinfo)

            # Test 1: uint8 input [0, 255] to target [0, 1] (normalization)
            # Expected: scale = (1-0)/(255-0) ≈ 0.00392, offset = 0 - 0*scale = 0.0
            input_mins = [0.0, 0.0, 0.0]
            input_maxs = [255.0, 255.0, 255.0]
            result = modelinfo.get_input_scales_offsets("input_normalized",
                                                        input_mins, input_maxs)
            self.assertTrue(result[0])
            scales = result[1]
            offsets = result[2]
            self.assertEqual(len(scales), 3)
            for i in range(3):
                self.assertAlmostEqual(scales[i], 1.0 / 255.0, 6)
                self.assertAlmostEqual(offsets[i], 0.0, 6)

            modelinfo.free()
        finally:
            if os.path.exists(temp_modelinfo):
                os.unlink(temp_modelinfo)
            if os.path.exists(model_filename):
                os.unlink(model_filename)

        # Create a modelinfo with a tensor that expects [-1, 1] range
        modelinfo_content_signed = """
[modelinfo]
version=1.0
group-id=test-model-signed

[input_signed]
dims=1,224,224,3
dir=input
type=int8
ranges=-1.0,1.0;-1.0,1.0;-1.0,1.0
"""

        with tempfile.NamedTemporaryFile(mode='w', suffix='.modelinfo',
                                         delete=False) as f:
            f.write(modelinfo_content_signed)
            temp_modelinfo = f.name

        try:
            model_filename = temp_modelinfo[:-10]
            modelinfo = GstAnalytics.ModelInfo.load(model_filename)
            self.assertIsNotNone(modelinfo)

            # Test 2: int8 input [-128, 127] to target [-1, 1]
            # Expected: scale = (1-(-1))/(127-(-128)) = 2/255 ≈ 0.00784
            #           offset = -1 - (-128)*scale = -1 + 128*0.00784 ≈ 0.00392
            input_mins = [-128.0, -128.0, -128.0]
            input_maxs = [127.0, 127.0, 127.0]
            result = modelinfo.get_input_scales_offsets("input_signed",
                                                        input_mins, input_maxs)
            self.assertTrue(result[0])
            scales = result[1]
            offsets = result[2]
            self.assertEqual(len(scales), 3)
            expected_scale = 2.0 / 255.0
            expected_offset = -1.0 - (-128.0) * expected_scale
            for i in range(3):
                self.assertAlmostEqual(scales[i], expected_scale, 6)
                self.assertAlmostEqual(offsets[i], expected_offset, 6)

            modelinfo.free()
        finally:
            if os.path.exists(temp_modelinfo):
                os.unlink(temp_modelinfo)
            if os.path.exists(model_filename):
                os.unlink(model_filename)

    def test_modelinfo_version_minor_upgrade_accepted(self):
        """Test that modelinfo with same major version but higher minor version is accepted"""
        import tempfile
        import os

        # Test case: Version 1.5 (same major version, higher minor version)
        # The version check should accept this since it's backward compatible
        modelinfo_content_1_5 = """[modelinfo]
version=1.5
group-id=test-model-v1-5

[input_tensor]
dims=1,224,224,3
dir=input
type=uint8

[output_tensor]
dims=1,1000
dir=output
type=float32
id=output_logits
"""

        with tempfile.NamedTemporaryFile(mode='w', suffix='.modelinfo',
                                         delete=False) as f:
            f.write(modelinfo_content_1_5)
            temp_modelinfo = f.name

        try:
            model_filename = temp_modelinfo[:-10]  # Remove '.modelinfo'
            # Load should succeed because version 1.5 is compatible with 1.0
            # (same major version)
            modelinfo = GstAnalytics.ModelInfo.load(model_filename)
            self.assertIsNotNone(modelinfo)

            # Verify version string
            version = modelinfo.get_version()
            self.assertEqual(version, "1.5")

            # Parse version string to verify major and minor components
            version_parts = version.split('.')
            self.assertEqual(len(version_parts), 2)
            major_version = int(version_parts[0])
            minor_version = int(version_parts[1])
            self.assertEqual(major_version, 1)
            self.assertEqual(minor_version, 5)

            modelinfo.free()
        finally:
            if os.path.exists(temp_modelinfo):
                os.unlink(temp_modelinfo)
            if os.path.exists(model_filename):
                os.unlink(model_filename)


class TestAnalyticsGroupMtdIterator(TestCase):
    def test_keypoint_group_iteration(self):
        """Test iterating through keypoints in a group"""
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        rmeta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        self.assertIsNotNone(rmeta)

        # Create a group of 3 keypoints
        positions = [10, 20, 30, 40, 50, 60]  # 3 keypoints (x, y pairs)
        (ret, group) = rmeta.add_keypoints_group("test-3-kp",
                                                 GstAnalytics.KeypointDimensions_2D,
                                                 positions, None, None, None)
        self.assertTrue(ret)
        self.assertIsNotNone(group)

        # Test iteration using __iter__
        keypoints = list(group)
        self.assertEqual(len(keypoints), 3)

        # Verify each keypoint can be accessed and has correct data
        for i, kp in enumerate(keypoints):
            success, x, y, z, dim = kp.get_position()
            self.assertTrue(success)
            self.assertEqual(x, positions[i * 2])
            self.assertEqual(y, positions[i * 2 + 1])
            self.assertEqual(z, 0)  # 2D keypoint
            self.assertEqual(dim, GstAnalytics.KeypointDimensions_2D)

    def test_keypoint_group_iteration_with_type_filter(self):
        """Test iterating through keypoints with type filtering"""
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        rmeta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        self.assertIsNotNone(rmeta)

        # Create a group of keypoints
        positions = [10, 20, 30, 40]  # 2 keypoints
        (ret, group) = rmeta.add_keypoints_group("test-2-kp",
                                                 GstAnalytics.KeypointDimensions_2D,
                                                 positions, None, None, None)
        self.assertTrue(ret)
        self.assertIsNotNone(group)

        # Iterate with type filter - only get KeypointMtd
        keypoints = list(group.iter_on_type(GstAnalytics.KeypointMtd))
        self.assertEqual(len(keypoints), 2)

        # Verify they're all KeypointMtd
        for kp in keypoints:
            self.assertIsInstance(kp, GstAnalytics.KeypointMtd)

    def test_keypoint_group_with_skeleton(self):
        """Test keypoint group with skeleton links"""
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        rmeta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        self.assertIsNotNone(rmeta)

        # Create keypoints with skeleton links
        positions = [10, 20, 30, 40, 50, 60]  # 3 keypoints
        confidences = [0.9, 0.8, 0.7]

        # Skeleton links as flat array [kp1_idx, kp2_idx, kp1_idx, kp2_idx, ...]
        # Creates links: 0<->1, 1<->2
        skeleton_pairs = [0, 1, 1, 2]

        (ret, group) = rmeta.add_keypoints_group("test-skeleton-3-kp",
                                                 GstAnalytics.KeypointDimensions_2D,
                                                 positions, confidences, None,
                                                 skeleton_pairs)
        self.assertTrue(ret)
        self.assertIsNotNone(group)

        # Verify group has 3 members
        member_count = group.get_member_count()
        self.assertEqual(member_count, 3)

        # Verify skeleton relations exist
        kp0 = group.get_member(0)
        self.assertTrue(kp0[0])  # success
        kp1 = group.get_member(1)
        self.assertTrue(kp1[0])  # success
        kp2 = group.get_member(2)
        self.assertTrue(kp2[0])  # success

        # Check if relation exists between first two keypoints (link 0<->1)
        has_relation, _ = rmeta.exist(kp0[1].id, kp1[1].id, 1,
                                      GstAnalytics.RelTypes.RELATE_TO)
        self.assertTrue(has_relation)

        # Check if relation exists between keypoints 1<->2
        has_relation, _ = rmeta.exist(kp1[1].id, kp2[1].id, 1,
                                      GstAnalytics.RelTypes.RELATE_TO)
        self.assertTrue(has_relation)

        # Verify confidences
        keypoints = list(group)
        for i, kp in enumerate(keypoints):
            success, conf = kp.get_confidence()
            self.assertTrue(success)
            self.assertAlmostEqual(conf, confidences[i], places=5)

    def test_keypoint_3d_group(self):
        """Test 3D keypoint group"""
        buf = Gst.Buffer()
        self.assertIsNotNone(buf)

        rmeta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        self.assertIsNotNone(rmeta)

        # Create 3D keypoints (2 keypoints with x, y, z)
        positions = [10, 20, 100, 30, 40, 200]  # 2 keypoints (x, y, z pairs)
        (ret, group) = rmeta.add_keypoints_group("test-3d-2-kp",
                                                 GstAnalytics.KeypointDimensions_3D,
                                                 positions, None, None, None)
        self.assertTrue(ret)
        self.assertIsNotNone(group)

        # Iterate and verify 3D coordinates
        keypoints = list(group)
        self.assertEqual(len(keypoints), 2)

        # First keypoint: (10, 20, 100)
        success, x, y, z, dim = keypoints[0].get_position()
        self.assertTrue(success)
        self.assertEqual(x, 10)
        self.assertEqual(y, 20)
        self.assertEqual(z, 100)
        self.assertEqual(dim, GstAnalytics.KeypointDimensions_3D)

        # Second keypoint: (30, 40, 200)
        success, x, y, z, dim = keypoints[1].get_position()
        self.assertTrue(success)
        self.assertEqual(x, 30)
        self.assertEqual(y, 40)
        self.assertEqual(z, 200)
        self.assertEqual(dim, GstAnalytics.KeypointDimensions_3D)


class TestAnalyticsGroupMtdSemanticTag(TestCase):
    def _make_group(self, tag):
        buf = Gst.Buffer()
        rmeta = GstAnalytics.buffer_add_analytics_relation_meta(buf)
        ret, group = rmeta.add_group_mtd_with_size(2)
        self.assertTrue(ret)
        group.set_semantic_tag(tag)
        return buf, group

    def test_has_semantic_tag_match(self):
        """has_semantic_tag returns True for the exact stored tag"""
        _, group = self._make_group("hand-21-kp")
        self.assertTrue(group.has_semantic_tag("hand-21-kp"))

    def test_has_semantic_tag_no_match(self):
        """has_semantic_tag returns False for a different tag"""
        _, group = self._make_group("hand-21-kp")
        self.assertFalse(group.has_semantic_tag("pose-17-kp"))

    def test_has_semantic_tag_unset(self):
        """has_semantic_tag returns False when no tag is set"""
        _, group = self._make_group(None)
        self.assertFalse(group.has_semantic_tag("hand-21-kp"))

    def test_semantic_tag_has_prefix_match(self):
        """semantic_tag_has_prefix returns True for a matching prefix"""
        _, group = self._make_group("posture/hand-21-kp")
        self.assertTrue(group.semantic_tag_has_prefix("posture/"))

    def test_semantic_tag_has_prefix_full_match(self):
        """semantic_tag_has_prefix returns True when prefix equals the full tag"""
        _, group = self._make_group("posture/hand-21-kp")
        self.assertTrue(group.semantic_tag_has_prefix("posture/hand-21-kp"))

    def test_semantic_tag_has_prefix_no_match(self):
        """semantic_tag_has_prefix returns False for a non-matching prefix"""
        _, group = self._make_group("posture/hand-21-kp")
        self.assertFalse(group.semantic_tag_has_prefix("keypoint/"))

    def test_semantic_tag_has_prefix_unset(self):
        """semantic_tag_has_prefix returns False when no tag is set"""
        _, group = self._make_group(None)
        self.assertFalse(group.semantic_tag_has_prefix("posture/"))



'''
Simple mixer element, accepts 320 x 240 RGBA at 30 fps
on any number of sinkpads.

Requires PIL (Python Imaging Library)

Example pipeline:

gst-launch-1.0 py_videomixer name=mixer ! videoconvert ! autovideosink \
        videotestsrc ! mixer. \
        videotestsrc pattern=ball ! mixer. \
        videotestsrc pattern=snow ! mixer.
'''

import gi

gi.require_version('Gst', '1.0')
gi.require_version('GstBase', '1.0')
gi.require_version('GObject', '2.0')
from gi.repository import Gst, GObject, GstBase

Gst.init_python()

try:
    from PIL import Image
except ImportError:
    Gst.error('py_videomixer requires PIL')
    raise

# Completely fixed input / output
ICAPS = Gst.Caps(Gst.Structure('video/x-raw',
                               format='RGBA',
                               width=320,
                               height=240,
                               framerate=Gst.Fraction(30, 1)))

OCAPS = Gst.Caps(Gst.Structure('video/x-raw',
                               format='RGBA',
                               width=320,
                               height=240,
                               framerate=Gst.Fraction(30, 1)))


class BlendData:
    def __init__(self, outimg):
        self.outimg = outimg
        self.pts = 0
        self.eos = True


class Videomixer(GstBase.Aggregator):
    __gstmetadata__ = ('Videomixer', 'Video/Mixer',
                       'Python video mixer', 'Mathieu Duponchelle')

    __gsttemplates__ = (
        Gst.PadTemplate.new_with_gtype("sink_%u",
                                       Gst.PadDirection.SINK,
                                       Gst.PadPresence.REQUEST,
                                       ICAPS,
                                       GstBase.AggregatorPad.__gtype__),
        Gst.PadTemplate.new_with_gtype("src",
                                       Gst.PadDirection.SRC,
                                       Gst.PadPresence.ALWAYS,
                                       OCAPS,
                                       GstBase.AggregatorPad.__gtype__)
    )

    def mix_buffers(self, agg, pad, bdata):
        buf = pad.pop_buffer()
        _, info = buf.map(Gst.MapFlags.READ)

        img = Image.frombuffer('RGBA', (320, 240), info.data, "raw", 'RGBA', 0, 1)

        bdata.outimg = Image.blend(bdata.outimg, img, alpha=0.5)
        bdata.pts = buf.pts

        # Need to ensure the PIL image has been released, or unmap will fail
        # with an outstanding memoryview buffer error
        del img

        buf.unmap(info)

        bdata.eos = False

        return True

    def do_aggregate(self, timeout):
        outimg = Image.new('RGBA', (320, 240), 0x00000000)

        bdata = BlendData(outimg)

        self.foreach_sink_pad(self.mix_buffers, bdata)

        data = bdata.outimg.tobytes()

        outbuf = Gst.Buffer.new_allocate(None, len(data), None)
        outbuf.fill(0, data)
        outbuf.pts = bdata.pts
        self.finish_buffer(outbuf)

        # We are EOS when no pad was ready to be aggregated,
        # this would obviously not work for live
        if bdata.eos:
            return Gst.FlowReturn.EOS

        return Gst.FlowReturn.OK


GObject.type_register(Videomixer)
__gstelementfactory__ = ("py_videomixer", Gst.Rank.NONE, Videomixer)
